@@ -8,6 +8,9 @@
 extern "C" {
 #endif
 
+unsigned isl_union_set_dim(__isl_keep isl_union_set *uset,
+	enum isl_dim_type type);
+
 __isl_constructor
 __isl_give isl_union_set *isl_union_set_from_basic_set(
 	__isl_take isl_basic_set *bset);
@@ -15,11 +18,14 @@ __isl_constructor
 __isl_give isl_union_set *isl_union_set_from_set(__isl_take isl_set *set);
 __isl_give isl_union_set *isl_union_set_empty(__isl_take isl_space *dim);
 __isl_give isl_union_set *isl_union_set_copy(__isl_keep isl_union_set *uset);
-void *isl_union_set_free(__isl_take isl_union_set *uset);
+__isl_null isl_union_set *isl_union_set_free(__isl_take isl_union_set *uset);
 
 isl_ctx *isl_union_set_get_ctx(__isl_keep isl_union_set *uset);
 __isl_give isl_space *isl_union_set_get_space(__isl_keep isl_union_set *uset);
 
+__isl_give isl_union_set *isl_union_set_reset_user(
+	__isl_take isl_union_set *uset);
+
 __isl_give isl_union_set *isl_union_set_universe(
 	__isl_take isl_union_set *uset);
 __isl_give isl_set *isl_union_set_params(__isl_take isl_union_set *uset);
@@ -33,6 +39,8 @@ __isl_give isl_union_set *isl_union_set_affine_hull(
 __isl_export
 __isl_give isl_union_set *isl_union_set_polyhedral_hull(
 	__isl_take isl_union_set *uset);
+__isl_give isl_union_set *isl_union_set_remove_redundancies(
+	__isl_take isl_union_set *uset);
 __isl_give isl_union_set *isl_union_set_simple_hull(
 	__isl_take isl_union_set *uset);
 __isl_export
@@ -70,32 +78,45 @@ __isl_give isl_union_set *isl_union_set_gist_params(
 __isl_export
 __isl_give isl_union_set *isl_union_set_apply(
 	__isl_take isl_union_set *uset, __isl_take isl_union_map *umap);
+__isl_give isl_union_set *isl_union_set_preimage_multi_aff(
+	__isl_take isl_union_set *uset, __isl_take isl_multi_aff *ma);
+__isl_give isl_union_set *isl_union_set_preimage_pw_multi_aff(
+	__isl_take isl_union_set *uset, __isl_take isl_pw_multi_aff *pma);
+__isl_give isl_union_set *isl_union_set_preimage_union_pw_multi_aff(
+	__isl_take isl_union_set *uset,
+	__isl_take isl_union_pw_multi_aff *upma);
 
-int isl_union_set_is_params(__isl_keep isl_union_set *uset);
+__isl_give isl_union_set *isl_union_set_project_out(
+	__isl_take isl_union_set *uset,
+	enum isl_dim_type type, unsigned first, unsigned n);
+
+isl_bool isl_union_set_is_params(__isl_keep isl_union_set *uset);
 __isl_export
-int isl_union_set_is_empty(__isl_keep isl_union_set *uset);
+isl_bool isl_union_set_is_empty(__isl_keep isl_union_set *uset);
 
 __isl_export
-int isl_union_set_is_subset(__isl_keep isl_union_set *uset1,
+isl_bool isl_union_set_is_subset(__isl_keep isl_union_set *uset1,
 	__isl_keep isl_union_set *uset2);
 __isl_export
-int isl_union_set_is_equal(__isl_keep isl_union_set *uset1,
+isl_bool isl_union_set_is_equal(__isl_keep isl_union_set *uset1,
+	__isl_keep isl_union_set *uset2);
+isl_bool isl_union_set_is_disjoint(__isl_keep isl_union_set *uset1,
 	__isl_keep isl_union_set *uset2);
 __isl_export
-int isl_union_set_is_strict_subset(__isl_keep isl_union_set *uset1,
+isl_bool isl_union_set_is_strict_subset(__isl_keep isl_union_set *uset1,
 	__isl_keep isl_union_set *uset2);
 
 int isl_union_set_n_set(__isl_keep isl_union_set *uset);
 __isl_export
-int isl_union_set_foreach_set(__isl_keep isl_union_set *uset,
-	int (*fn)(__isl_take isl_set *set, void *user), void *user);
+isl_stat isl_union_set_foreach_set(__isl_keep isl_union_set *uset,
+	isl_stat (*fn)(__isl_take isl_set *set, void *user), void *user);
 __isl_give int isl_union_set_contains(__isl_keep isl_union_set *uset,
 	__isl_keep isl_space *dim);
 __isl_give isl_set *isl_union_set_extract_set(__isl_keep isl_union_set *uset,
 	__isl_take isl_space *dim);
 __isl_give isl_set *isl_set_from_union_set(__isl_take isl_union_set *uset);
-int isl_union_set_foreach_point(__isl_keep isl_union_set *uset,
-	int (*fn)(__isl_take isl_point *pnt, void *user), void *user);
+isl_stat isl_union_set_foreach_point(__isl_keep isl_union_set *uset,
+	isl_stat (*fn)(__isl_take isl_point *pnt, void *user), void *user);
 
 __isl_give isl_basic_set *isl_union_set_sample(__isl_take isl_union_set *uset);
 
@@ -120,14 +141,18 @@ __isl_give isl_union_set *isl_union_set_read_from_file(isl_ctx *ctx,
 __isl_constructor
 __isl_give isl_union_set *isl_union_set_read_from_str(isl_ctx *ctx,
 	const char *str);
+__isl_give char *isl_union_set_to_str(__isl_keep isl_union_set *uset);
 __isl_give isl_printer *isl_printer_print_union_set(__isl_take isl_printer *p,
 	__isl_keep isl_union_set *uset);
 void isl_union_set_dump(__isl_keep isl_union_set *uset);
 
+ISL_DECLARE_LIST_FN(union_set)
+
+__isl_give isl_union_set *isl_union_set_list_union(
+	__isl_take isl_union_set_list *list);
+
 #if defined(__cplusplus)
 }
 #endif
 
-#include <isl/dim.h>
-
 #endif
