@@ -14,7 +14,7 @@
 
 #include <isl_ctx_private.h>
 #include <isl_map_private.h>
-#include <isl/seq.h>
+#include <isl_seq.h>
 #include <isl/set.h>
 #include <isl/lp.h>
 #include <isl/map.h>
@@ -22,6 +22,7 @@
 #include "isl_sample.h"
 #include "isl_tab.h"
 #include <isl_mat_private.h>
+#include <isl_vec_private.h>
 
 struct isl_basic_map *isl_basic_map_implicit_equalities(
 						struct isl_basic_map *bmap)
@@ -473,10 +474,9 @@ static __isl_give isl_basic_set *extend_affine_hull(struct isl_tab *tab,
 		}
 		if (j == hull->n_eq)
 			break;
-		if (tab->samples)
-			tab = isl_tab_add_sample(tab, isl_vec_copy(sample));
-		if (!tab)
-			goto error;
+		if (tab->samples &&
+		    isl_tab_add_sample(tab, isl_vec_copy(sample)) < 0)
+			hull = isl_basic_set_free(hull);
 		if (bset)
 			hull = add_adjacent_points(hull, isl_vec_copy(sample),
 						    bset);
@@ -520,6 +520,7 @@ static __isl_give isl_basic_map *isl_basic_map_drop_constraints_involving(
 		isl_basic_map_drop_inequality(bmap, i);
 	}
 
+	bmap = isl_basic_map_add_known_div_constraints(bmap);
 	return bmap;
 }
 
@@ -542,8 +543,11 @@ __isl_give isl_basic_map *isl_basic_map_drop_constraints_not_involving_dims(
 	int i;
 	unsigned dim;
 
-	if (n == 0)
-		return isl_basic_map_set_to_empty(bmap);
+	if (n == 0) {
+		isl_space *space = isl_basic_map_get_space(bmap);
+		isl_basic_map_free(bmap);
+		return isl_basic_map_universe(space);
+	}
 	bmap = isl_basic_map_cow(bmap);
 	if (!bmap)
 		return NULL;
@@ -567,6 +571,7 @@ __isl_give isl_basic_map *isl_basic_map_drop_constraints_not_involving_dims(
 		isl_basic_map_drop_inequality(bmap, i);
 	}
 
+	bmap = isl_basic_map_add_known_div_constraints(bmap);
 	return bmap;
 }
 
@@ -1000,11 +1005,11 @@ static struct isl_basic_set *uset_affine_hull(struct isl_basic_set *bset)
 	if (!cone)
 		goto error;
 	if (cone->n_eq == 0) {
-		struct isl_basic_set *hull;
+		isl_space *space;
+		space = isl_basic_set_get_space(bset);
 		isl_basic_set_free(cone);
-		hull = isl_basic_set_universe_like(bset);
 		isl_basic_set_free(bset);
-		return hull;
+		return isl_basic_set_universe(space);
 	}
 
 	if (cone->n_eq < isl_basic_set_total_dim(cone))
@@ -1142,6 +1147,19 @@ __isl_give isl_set *isl_set_detect_equalities(__isl_take isl_set *set)
 	return (isl_set *)isl_map_detect_equalities((isl_map *)set);
 }
 
+/* Return the superset of "bmap" described by the equalities
+ * satisfied by "bmap" that are already known.
+ */
+__isl_give isl_basic_map *isl_basic_map_plain_affine_hull(
+	__isl_take isl_basic_map *bmap)
+{
+	bmap = isl_basic_map_cow(bmap);
+	if (bmap)
+		isl_basic_map_free_inequality(bmap, bmap->n_ineq);
+	bmap = isl_basic_map_finalize(bmap);
+	return bmap;
+}
+
 /* After computing the rational affine hull (by detecting the implicit
  * equalities), we compute the additional equalities satisfied by
  * the integer points (if any) and add the original equalities back in.
@@ -1149,10 +1167,7 @@ __isl_give isl_set *isl_set_detect_equalities(__isl_take isl_set *set)
 struct isl_basic_map *isl_basic_map_affine_hull(struct isl_basic_map *bmap)
 {
 	bmap = isl_basic_map_detect_equalities(bmap);
-	bmap = isl_basic_map_cow(bmap);
-	if (bmap)
-		isl_basic_map_free_inequality(bmap, bmap->n_ineq);
-	bmap = isl_basic_map_finalize(bmap);
+	bmap = isl_basic_map_plain_affine_hull(bmap);
 	return bmap;
 }
 
@@ -1331,6 +1346,18 @@ static __isl_give isl_set *isl_set_local_affine_hull(__isl_take isl_set *set)
 	return isl_map_local_affine_hull(set);
 }
 
+/* Return an empty basic map living in the same space as "map".
+ */
+static __isl_give isl_basic_map *replace_map_by_empty_basic_map(
+	__isl_take isl_map *map)
+{
+	isl_space *space;
+
+	space = isl_map_get_space(map);
+	isl_map_free(map);
+	return isl_basic_map_empty(space);
+}
+
 /* Compute the affine hull of "map".
  *
  * We first compute the affine hull of each basic map separately.
@@ -1366,11 +1393,8 @@ __isl_give isl_basic_map *isl_map_affine_hull(__isl_take isl_map *map)
 	if (!map)
 		return NULL;
 
-	if (map->n == 0) {
-		hull = isl_basic_map_empty_like_map(map);
-		isl_map_free(map);
-		return hull;
-	}
+	if (map->n == 0)
+		return replace_map_by_empty_basic_map(map);
 
 	model = isl_basic_map_copy(map->p[0]);
 	set = isl_map_underlying_set(map);
