@@ -1,5 +1,13 @@
+/*
+ * Copyright 2012-2013 Ecole Normale Superieure
+ *
+ * Use of this software is governed by the MIT license
+ *
+ * Written by Sven Verdoolaege,
+ * Ecole Normale Superieure, 45 rue d’Ulm, 75230 Paris, France
+ */
+
 #include <isl_ast_private.h>
-#include <isl_list_private.h>
 
 #undef BASE
 #define BASE ast_expr
@@ -76,7 +84,8 @@ __isl_give isl_ast_print_options *isl_ast_print_options_copy(
 	return options;
 }
 
-void *isl_ast_print_options_free(__isl_take isl_ast_print_options *options)
+__isl_null isl_ast_print_options *isl_ast_print_options_free(
+	__isl_take isl_ast_print_options *options)
 {
 	if (!options)
 		return NULL;
@@ -154,7 +163,7 @@ __isl_give isl_ast_expr *isl_ast_expr_dup(__isl_keep isl_ast_expr *expr)
 	ctx = isl_ast_expr_get_ctx(expr);
 	switch (expr->type) {
 	case isl_ast_expr_int:
-		dup = isl_ast_expr_alloc_int(ctx, expr->u.i);
+		dup = isl_ast_expr_from_val(isl_val_copy(expr->u.v));
 		break;
 	case isl_ast_expr_id:
 		dup = isl_ast_expr_from_id(isl_id_copy(expr->u.id));
@@ -189,7 +198,7 @@ __isl_give isl_ast_expr *isl_ast_expr_cow(__isl_take isl_ast_expr *expr)
 	return isl_ast_expr_dup(expr);
 }
 
-void *isl_ast_expr_free(__isl_take isl_ast_expr *expr)
+__isl_null isl_ast_expr *isl_ast_expr_free(__isl_take isl_ast_expr *expr)
 {
 	int i;
 
@@ -203,14 +212,15 @@ void *isl_ast_expr_free(__isl_take isl_ast_expr *expr)
 
 	switch (expr->type) {
 	case isl_ast_expr_int:
-		isl_int_clear(expr->u.i);
+		isl_val_free(expr->u.v);
 		break;
 	case isl_ast_expr_id:
 		isl_id_free(expr->u.id);
 		break;
 	case isl_ast_expr_op:
-		for (i = 0; i < expr->u.op.n_arg; ++i)
-			isl_ast_expr_free(expr->u.op.args[i]);
+		if (expr->u.op.args)
+			for (i = 0; i < expr->u.op.n_arg; ++i)
+				isl_ast_expr_free(expr->u.op.args[i]);
 		free(expr->u.op.args);
 		break;
 	case isl_ast_expr_error:
@@ -231,15 +241,16 @@ enum isl_ast_expr_type isl_ast_expr_get_type(__isl_keep isl_ast_expr *expr)
 	return expr ? expr->type : isl_ast_expr_error;
 }
 
-int isl_ast_expr_get_int(__isl_keep isl_ast_expr *expr, isl_int *v)
+/* Return the integer value represented by "expr".
+ */
+__isl_give isl_val *isl_ast_expr_get_val(__isl_keep isl_ast_expr *expr)
 {
 	if (!expr)
-		return -1;
+		return NULL;
 	if (expr->type != isl_ast_expr_int)
 		isl_die(isl_ast_expr_get_ctx(expr), isl_error_invalid,
-			"expression not an int", return -1);
-	isl_int_set(*v, expr->u.i);
-	return 0;
+			"expression not an int", return NULL);
+	return isl_val_copy(expr->u.v);
 }
 
 __isl_give isl_id *isl_ast_expr_get_id(__isl_keep isl_ast_expr *expr)
@@ -312,6 +323,46 @@ error:
 	return isl_ast_expr_free(expr);
 }
 
+/* Is "expr1" equal to "expr2"?
+ */
+isl_bool isl_ast_expr_is_equal(__isl_keep isl_ast_expr *expr1,
+	__isl_keep isl_ast_expr *expr2)
+{
+	int i;
+
+	if (!expr1 || !expr2)
+		return isl_bool_error;
+
+	if (expr1 == expr2)
+		return isl_bool_true;
+	if (expr1->type != expr2->type)
+		return isl_bool_false;
+	switch (expr1->type) {
+	case isl_ast_expr_int:
+		return isl_val_eq(expr1->u.v, expr2->u.v);
+	case isl_ast_expr_id:
+		return expr1->u.id == expr2->u.id;
+	case isl_ast_expr_op:
+		if (expr1->u.op.op != expr2->u.op.op)
+			return isl_bool_false;
+		if (expr1->u.op.n_arg != expr2->u.op.n_arg)
+			return isl_bool_false;
+		for (i = 0; i < expr1->u.op.n_arg; ++i) {
+			isl_bool equal;
+			equal = isl_ast_expr_is_equal(expr1->u.op.args[i],
+							expr2->u.op.args[i]);
+			if (equal < 0 || !equal)
+				return equal;
+		}
+		return 1;
+	case isl_ast_expr_error:
+		return isl_bool_error;
+	}
+
+	isl_die(isl_ast_expr_get_ctx(expr1), isl_error_internal,
+		"unhandled case", return isl_bool_error);
+}
+
 /* Create a new operation expression of operation type "op",
  * with "n_arg" as yet unspecified arguments.
  */
@@ -332,7 +383,7 @@ __isl_give isl_ast_expr *isl_ast_expr_alloc_op(isl_ctx *ctx,
 	expr->u.op.n_arg = n_arg;
 	expr->u.op.args = isl_calloc_array(ctx, isl_ast_expr *, n_arg);
 
-	if (!expr->u.op.args)
+	if (n_arg && !expr->u.op.args)
 		return isl_ast_expr_free(expr);
 
 	return expr;
@@ -351,7 +402,7 @@ __isl_give isl_ast_expr *isl_ast_expr_from_id(__isl_take isl_id *id)
 	ctx = isl_id_get_ctx(id);
 	expr = isl_calloc_type(ctx, isl_ast_expr);
 	if (!expr)
-		return isl_id_free(id);
+		goto error;
 
 	expr->ctx = ctx;
 	isl_ctx_ref(ctx);
@@ -360,6 +411,9 @@ __isl_give isl_ast_expr *isl_ast_expr_from_id(__isl_take isl_id *id)
 	expr->u.id = id;
 
 	return expr;
+error:
+	isl_id_free(id);
+	return NULL;
 }
 
 /* Create a new integer expression representing "i".
@@ -376,37 +430,48 @@ __isl_give isl_ast_expr *isl_ast_expr_alloc_int_si(isl_ctx *ctx, int i)
 	isl_ctx_ref(ctx);
 	expr->ref = 1;
 	expr->type = isl_ast_expr_int;
-
-	isl_int_init(expr->u.i);
-	isl_int_set_si(expr->u.i, i);
+	expr->u.v = isl_val_int_from_si(ctx, i);
+	if (!expr->u.v)
+		return isl_ast_expr_free(expr);
 
 	return expr;
 }
 
-/* Create a new integer expression representing "i".
+/* Create a new integer expression representing "v".
  */
-__isl_give isl_ast_expr *isl_ast_expr_alloc_int(isl_ctx *ctx, isl_int i)
+__isl_give isl_ast_expr *isl_ast_expr_from_val(__isl_take isl_val *v)
 {
+	isl_ctx *ctx;
 	isl_ast_expr *expr;
 
+	if (!v)
+		return NULL;
+	if (!isl_val_is_int(v))
+		isl_die(isl_val_get_ctx(v), isl_error_invalid,
+			"expecting integer value", goto error);
+
+	ctx = isl_val_get_ctx(v);
 	expr = isl_calloc_type(ctx, isl_ast_expr);
 	if (!expr)
-		return NULL;
+		goto error;
 
 	expr->ctx = ctx;
 	isl_ctx_ref(ctx);
 	expr->ref = 1;
 	expr->type = isl_ast_expr_int;
-
-	isl_int_init(expr->u.i);
-	isl_int_set(expr->u.i, i);
+	expr->u.v = v;
 
 	return expr;
+error:
+	isl_val_free(v);
+	return NULL;
 }
 
-/* Create an expression representing the negation of "arg".
+/* Create an expression representing the unary operation "type" applied to
+ * "arg".
  */
-__isl_give isl_ast_expr *isl_ast_expr_neg(__isl_take isl_ast_expr *arg)
+__isl_give isl_ast_expr *isl_ast_expr_alloc_unary(enum isl_ast_op_type type,
+	__isl_take isl_ast_expr *arg)
 {
 	isl_ctx *ctx;
 	isl_ast_expr *expr = NULL;
@@ -415,7 +480,7 @@ __isl_give isl_ast_expr *isl_ast_expr_neg(__isl_take isl_ast_expr *arg)
 		return NULL;
 
 	ctx = isl_ast_expr_get_ctx(arg);
-	expr = isl_ast_expr_alloc_op(ctx, isl_ast_op_minus, 1);
+	expr = isl_ast_expr_alloc_op(ctx, type, 1);
 	if (!expr)
 		goto error;
 
@@ -427,6 +492,29 @@ error:
 	return NULL;
 }
 
+/* Create an expression representing the negation of "arg".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_neg(__isl_take isl_ast_expr *arg)
+{
+	return isl_ast_expr_alloc_unary(isl_ast_op_minus, arg);
+}
+
+/* Create an expression representing the address of "expr".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_address_of(__isl_take isl_ast_expr *expr)
+{
+	if (!expr)
+		return NULL;
+
+	if (isl_ast_expr_get_type(expr) != isl_ast_expr_op ||
+	    isl_ast_expr_get_op_type(expr) != isl_ast_op_access)
+		isl_die(isl_ast_expr_get_ctx(expr), isl_error_invalid,
+			"can only take address of access expressions",
+			return isl_ast_expr_free(expr));
+
+	return isl_ast_expr_alloc_unary(isl_ast_op_address_of, expr);
+}
+
 /* Create an expression representing the binary operation "type"
  * applied to "expr1" and "expr2".
  */
@@ -486,6 +574,26 @@ __isl_give isl_ast_expr *isl_ast_expr_div(__isl_take isl_ast_expr *expr1,
 	return isl_ast_expr_alloc_binary(isl_ast_op_div, expr1, expr2);
 }
 
+/* Create an expression representing the quotient of the integer
+ * division of "expr1" by "expr2", where "expr1" is known to be
+ * non-negative.
+ */
+__isl_give isl_ast_expr *isl_ast_expr_pdiv_q(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_pdiv_q, expr1, expr2);
+}
+
+/* Create an expression representing the remainder of the integer
+ * division of "expr1" by "expr2", where "expr1" is known to be
+ * non-negative.
+ */
+__isl_give isl_ast_expr *isl_ast_expr_pdiv_r(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_pdiv_r, expr1, expr2);
+}
+
 /* Create an expression representing the conjunction of "expr1" and "expr2".
  */
 __isl_give isl_ast_expr *isl_ast_expr_and(__isl_take isl_ast_expr *expr1,
@@ -494,6 +602,15 @@ __isl_give isl_ast_expr *isl_ast_expr_and(__isl_take isl_ast_expr *expr1,
 	return isl_ast_expr_alloc_binary(isl_ast_op_and, expr1, expr2);
 }
 
+/* Create an expression representing the conjunction of "expr1" and "expr2",
+ * where "expr2" is evaluated only if "expr1" is evaluated to true.
+ */
+__isl_give isl_ast_expr *isl_ast_expr_and_then(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_and_then, expr1, expr2);
+}
+
 /* Create an expression representing the disjunction of "expr1" and "expr2".
  */
 __isl_give isl_ast_expr *isl_ast_expr_or(__isl_take isl_ast_expr *expr1,
@@ -502,6 +619,167 @@ __isl_give isl_ast_expr *isl_ast_expr_or(__isl_take isl_ast_expr *expr1,
 	return isl_ast_expr_alloc_binary(isl_ast_op_or, expr1, expr2);
 }
 
+/* Create an expression representing the disjunction of "expr1" and "expr2",
+ * where "expr2" is evaluated only if "expr1" is evaluated to false.
+ */
+__isl_give isl_ast_expr *isl_ast_expr_or_else(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_or_else, expr1, expr2);
+}
+
+/* Create an expression representing "expr1" less than or equal to "expr2".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_le(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_le, expr1, expr2);
+}
+
+/* Create an expression representing "expr1" less than "expr2".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_lt(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_lt, expr1, expr2);
+}
+
+/* Create an expression representing "expr1" greater than or equal to "expr2".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_ge(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_ge, expr1, expr2);
+}
+
+/* Create an expression representing "expr1" greater than "expr2".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_gt(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_gt, expr1, expr2);
+}
+
+/* Create an expression representing "expr1" equal to "expr2".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_eq(__isl_take isl_ast_expr *expr1,
+	__isl_take isl_ast_expr *expr2)
+{
+	return isl_ast_expr_alloc_binary(isl_ast_op_eq, expr1, expr2);
+}
+
+/* Create an expression of type "type" with as arguments "arg0" followed
+ * by "arguments".
+ */
+static __isl_give isl_ast_expr *ast_expr_with_arguments(
+	enum isl_ast_op_type type, __isl_take isl_ast_expr *arg0,
+	__isl_take isl_ast_expr_list *arguments)
+{
+	int i, n;
+	isl_ctx *ctx;
+	isl_ast_expr *res = NULL;
+
+	if (!arg0 || !arguments)
+		goto error;
+
+	ctx = isl_ast_expr_get_ctx(arg0);
+	n = isl_ast_expr_list_n_ast_expr(arguments);
+	res = isl_ast_expr_alloc_op(ctx, type, 1 + n);
+	if (!res)
+		goto error;
+	for (i = 0; i < n; ++i) {
+		isl_ast_expr *arg;
+		arg = isl_ast_expr_list_get_ast_expr(arguments, i);
+		res->u.op.args[1 + i] = arg;
+		if (!arg)
+			goto error;
+	}
+	res->u.op.args[0] = arg0;
+
+	isl_ast_expr_list_free(arguments);
+	return res;
+error:
+	isl_ast_expr_free(arg0);
+	isl_ast_expr_list_free(arguments);
+	isl_ast_expr_free(res);
+	return NULL;
+}
+
+/* Create an expression representing an access to "array" with index
+ * expressions "indices".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_access(__isl_take isl_ast_expr *array,
+	__isl_take isl_ast_expr_list *indices)
+{
+	return ast_expr_with_arguments(isl_ast_op_access, array, indices);
+}
+
+/* Create an expression representing a call to "function" with argument
+ * expressions "arguments".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_call(__isl_take isl_ast_expr *function,
+	__isl_take isl_ast_expr_list *arguments)
+{
+	return ast_expr_with_arguments(isl_ast_op_call, function, arguments);
+}
+
+/* For each subexpression of "expr" of type isl_ast_expr_id,
+ * if it appears in "id2expr", then replace it by the corresponding
+ * expression.
+ */
+__isl_give isl_ast_expr *isl_ast_expr_substitute_ids(
+	__isl_take isl_ast_expr *expr, __isl_take isl_id_to_ast_expr *id2expr)
+{
+	int i;
+	isl_id *id;
+
+	if (!expr || !id2expr)
+		goto error;
+
+	switch (expr->type) {
+	case isl_ast_expr_int:
+		break;
+	case isl_ast_expr_id:
+		if (!isl_id_to_ast_expr_has(id2expr, expr->u.id))
+			break;
+		id = isl_id_copy(expr->u.id);
+		isl_ast_expr_free(expr);
+		expr = isl_id_to_ast_expr_get(id2expr, id);
+		break;
+	case isl_ast_expr_op:
+		for (i = 0; i < expr->u.op.n_arg; ++i) {
+			isl_ast_expr *arg;
+			arg = isl_ast_expr_copy(expr->u.op.args[i]);
+			arg = isl_ast_expr_substitute_ids(arg,
+					    isl_id_to_ast_expr_copy(id2expr));
+			if (arg == expr->u.op.args[i]) {
+				isl_ast_expr_free(arg);
+				continue;
+			}
+			if (!arg)
+				expr = isl_ast_expr_free(expr);
+			expr = isl_ast_expr_cow(expr);
+			if (!expr) {
+				isl_ast_expr_free(arg);
+				break;
+			}
+			isl_ast_expr_free(expr->u.op.args[i]);
+			expr->u.op.args[i] = arg;
+		}
+		break;
+	case isl_ast_expr_error:
+		expr = isl_ast_expr_free(expr);
+		break;
+	}
+
+	isl_id_to_ast_expr_free(id2expr);
+	return expr;
+error:
+	isl_ast_expr_free(expr);
+	isl_id_to_ast_expr_free(id2expr);
+	return NULL;
+}
+
 isl_ctx *isl_ast_node_get_ctx(__isl_keep isl_ast_node *node)
 {
 	return node ? node->ctx : NULL;
@@ -566,13 +844,42 @@ __isl_give isl_ast_node *isl_ast_node_alloc_for(__isl_take isl_id *id)
 	ctx = isl_id_get_ctx(id);
 	node = isl_ast_node_alloc(ctx, isl_ast_node_for);
 	if (!node)
-		return NULL;
+		goto error;
 
 	node->u.f.iterator = isl_ast_expr_from_id(id);
 	if (!node->u.f.iterator)
 		return isl_ast_node_free(node);
 
 	return node;
+error:
+	isl_id_free(id);
+	return NULL;
+}
+
+/* Create a mark node, marking "node" with "id".
+ */
+__isl_give isl_ast_node *isl_ast_node_alloc_mark(__isl_take isl_id *id,
+	__isl_take isl_ast_node *node)
+{
+	isl_ctx *ctx;
+	isl_ast_node *mark;
+
+	if (!id || !node)
+		goto error;
+
+	ctx = isl_id_get_ctx(id);
+	mark = isl_ast_node_alloc(ctx, isl_ast_node_mark);
+	if (!mark)
+		goto error;
+
+	mark->u.m.mark = id;
+	mark->u.m.node = node;
+
+	return mark;
+error:
+	isl_id_free(id);
+	isl_ast_node_free(node);
+	return NULL;
 }
 
 /* Create a user node evaluating "expr".
@@ -684,6 +991,12 @@ __isl_give isl_ast_node *isl_ast_node_dup(__isl_keep isl_ast_node *node)
 		if (!dup->u.b.children)
 			return isl_ast_node_free(dup);
 		break;
+	case isl_ast_node_mark:
+		dup->u.m.mark = isl_id_copy(node->u.m.mark);
+		dup->u.m.node = isl_ast_node_copy(node->u.m.node);
+		if (!dup->u.m.mark || !dup->u.m.node)
+			return isl_ast_node_free(dup);
+		break;
 	case isl_ast_node_user:
 		dup->u.e.expr = isl_ast_expr_copy(node->u.e.expr);
 		if (!dup->u.e.expr)
@@ -707,7 +1020,7 @@ __isl_give isl_ast_node *isl_ast_node_cow(__isl_take isl_ast_node *node)
 	return isl_ast_node_dup(node);
 }
 
-void *isl_ast_node_free(__isl_take isl_ast_node *node)
+__isl_null isl_ast_node *isl_ast_node_free(__isl_take isl_ast_node *node)
 {
 	if (!node)
 		return NULL;
@@ -731,6 +1044,10 @@ void *isl_ast_node_free(__isl_take isl_ast_node *node)
 	case isl_ast_node_block:
 		isl_ast_node_list_free(node->u.b.children);
 		break;
+	case isl_ast_node_mark:
+		isl_id_free(node->u.m.mark);
+		isl_ast_node_free(node->u.m.node);
+		break;
 	case isl_ast_node_user:
 		isl_ast_expr_free(node->u.e.expr);
 		break;
@@ -790,13 +1107,13 @@ __isl_give isl_ast_node *isl_ast_node_for_mark_degenerate(
 	return node;
 }
 
-int isl_ast_node_for_is_degenerate(__isl_keep isl_ast_node *node)
+isl_bool isl_ast_node_for_is_degenerate(__isl_keep isl_ast_node *node)
 {
 	if (!node)
-		return -1;
+		return isl_bool_error;
 	if (node->type != isl_ast_node_for)
 		isl_die(isl_ast_node_get_ctx(node), isl_error_invalid,
-			"not a for node", return -1);
+			"not a for node", return isl_bool_error);
 	return node->u.f.degenerate;
 }
 
@@ -896,14 +1213,14 @@ __isl_give isl_ast_node *isl_ast_node_if_get_then(
 	return isl_ast_node_copy(node->u.i.then);
 }
 
-int isl_ast_node_if_has_else(
+isl_bool isl_ast_node_if_has_else(
 	__isl_keep isl_ast_node *node)
 {
 	if (!node)
-		return -1;
+		return isl_bool_error;
 	if (node->type != isl_ast_node_if)
 		isl_die(isl_ast_node_get_ctx(node), isl_error_invalid,
-			"not an if node", return -1);
+			"not an if node", return isl_bool_error);
 	return node->u.i.else_node != NULL;
 }
 
@@ -945,10 +1262,40 @@ __isl_give isl_ast_expr *isl_ast_node_user_get_expr(
 {
 	if (!node)
 		return NULL;
+	if (node->type != isl_ast_node_user)
+		isl_die(isl_ast_node_get_ctx(node), isl_error_invalid,
+			"not a user node", return NULL);
 
 	return isl_ast_expr_copy(node->u.e.expr);
 }
 
+/* Return the mark identifier of the mark node "node".
+ */
+__isl_give isl_id *isl_ast_node_mark_get_id(__isl_keep isl_ast_node *node)
+{
+	if (!node)
+		return NULL;
+	if (node->type != isl_ast_node_mark)
+		isl_die(isl_ast_node_get_ctx(node), isl_error_invalid,
+			"not a mark node", return NULL);
+
+	return isl_id_copy(node->u.m.mark);
+}
+
+/* Return the node marked by mark node "node".
+ */
+__isl_give isl_ast_node *isl_ast_node_mark_get_node(
+	__isl_keep isl_ast_node *node)
+{
+	if (!node)
+		return NULL;
+	if (node->type != isl_ast_node_mark)
+		isl_die(isl_ast_node_get_ctx(node), isl_error_invalid,
+			"not a mark node", return NULL);
+
+	return isl_ast_node_copy(node->u.m.node);
+}
+
 __isl_give isl_id *isl_ast_node_get_annotation(__isl_keep isl_ast_node *node)
 {
 	return node ? isl_id_copy(node->annotation) : NULL;
@@ -987,12 +1334,15 @@ static char *op_str[] = {
 	[isl_ast_op_mul] = "*",
 	[isl_ast_op_pdiv_q] = "/",
 	[isl_ast_op_pdiv_r] = "%",
+	[isl_ast_op_zdiv_r] = "%",
 	[isl_ast_op_div] = "/",
 	[isl_ast_op_eq] = "==",
 	[isl_ast_op_le] = "<=",
 	[isl_ast_op_ge] = ">=",
 	[isl_ast_op_lt] = "<",
-	[isl_ast_op_gt] = ">"
+	[isl_ast_op_gt] = ">",
+	[isl_ast_op_member] = ".",
+	[isl_ast_op_address_of] = "&"
 };
 
 /* Precedence in C of the various operators.
@@ -1014,6 +1364,7 @@ static int op_prec[] = {
 	[isl_ast_op_fdiv_q] = 2,
 	[isl_ast_op_pdiv_q] = 5,
 	[isl_ast_op_pdiv_r] = 5,
+	[isl_ast_op_zdiv_r] = 5,
 	[isl_ast_op_cond] = 15,
 	[isl_ast_op_select] = 15,
 	[isl_ast_op_eq] = 9,
@@ -1021,7 +1372,10 @@ static int op_prec[] = {
 	[isl_ast_op_ge] = 8,
 	[isl_ast_op_lt] = 8,
 	[isl_ast_op_gt] = 8,
-	[isl_ast_op_call] = 2
+	[isl_ast_op_call] = 2,
+	[isl_ast_op_access] = 2,
+	[isl_ast_op_member] = 2,
+	[isl_ast_op_address_of] = 3
 };
 
 /* Is the operator left-to-right associative?
@@ -1041,6 +1395,7 @@ static int op_left[] = {
 	[isl_ast_op_fdiv_q] = 1,
 	[isl_ast_op_pdiv_q] = 1,
 	[isl_ast_op_pdiv_r] = 1,
+	[isl_ast_op_zdiv_r] = 1,
 	[isl_ast_op_cond] = 0,
 	[isl_ast_op_select] = 0,
 	[isl_ast_op_eq] = 1,
@@ -1048,7 +1403,10 @@ static int op_left[] = {
 	[isl_ast_op_ge] = 1,
 	[isl_ast_op_lt] = 1,
 	[isl_ast_op_gt] = 1,
-	[isl_ast_op_call] = 1
+	[isl_ast_op_call] = 1,
+	[isl_ast_op_access] = 1,
+	[isl_ast_op_member] = 1,
+	[isl_ast_op_address_of] = 0
 };
 
 static int is_and(enum isl_ast_op_type op)
@@ -1068,7 +1426,9 @@ static int is_add_sub(enum isl_ast_op_type op)
 
 static int is_div_mod(enum isl_ast_op_type op)
 {
-	return op == isl_ast_op_div || op == isl_ast_op_pdiv_r;
+	return op == isl_ast_op_div ||
+	       op == isl_ast_op_pdiv_r ||
+	       op == isl_ast_op_zdiv_r;
 }
 
 /* Do we need/want parentheses around "expr" as a subexpression of
@@ -1171,6 +1531,25 @@ static __isl_give isl_printer *print_call(__isl_take isl_printer *p,
 	return p;
 }
 
+/* Print an array access "expr".
+ *
+ * The first argument represents the array being accessed.
+ */
+static __isl_give isl_printer *print_access(__isl_take isl_printer *p,
+	__isl_keep isl_ast_expr *expr)
+{
+	int i = 0;
+
+	p = isl_printer_print_ast_expr(p, expr->u.op.args[0]);
+	for (i = 1; i < expr->u.op.n_arg; ++i) {
+		p = isl_printer_print_str(p, "[");
+		p = isl_printer_print_ast_expr(p, expr->u.op.args[i]);
+		p = isl_printer_print_str(p, "]");
+	}
+
+	return p;
+}
+
 /* Print "expr" to "p".
  *
  * If we are printing in isl format, then we also print an indication
@@ -1190,6 +1569,10 @@ __isl_give isl_printer *isl_printer_print_ast_expr(__isl_take isl_printer *p,
 			p = print_call(p, expr);
 			break;
 		}
+		if (expr->u.op.op == isl_ast_op_access) {
+			p = print_access(p, expr);
+			break;
+		}
 		if (expr->u.op.n_arg == 1) {
 			p = isl_printer_print_str(p, op_str[expr->u.op.op]);
 			p = print_sub_expr(p, expr->u.op.op,
@@ -1223,16 +1606,18 @@ __isl_give isl_printer *isl_printer_print_ast_expr(__isl_take isl_printer *p,
 				"operation should have two arguments",
 				goto error);
 		p = print_sub_expr(p, expr->u.op.op, expr->u.op.args[0], 1);
-		p = isl_printer_print_str(p, " ");
+		if (expr->u.op.op != isl_ast_op_member)
+			p = isl_printer_print_str(p, " ");
 		p = isl_printer_print_str(p, op_str[expr->u.op.op]);
-		p = isl_printer_print_str(p, " ");
+		if (expr->u.op.op != isl_ast_op_member)
+			p = isl_printer_print_str(p, " ");
 		p = print_sub_expr(p, expr->u.op.op, expr->u.op.args[1], 0);
 		break;
 	case isl_ast_expr_id:
 		p = isl_printer_print_str(p, isl_id_get_name(expr->u.id));
 		break;
 	case isl_ast_expr_int:
-		p = isl_printer_print_isl_int(p, expr->u.i);
+		p = isl_printer_print_val(p, expr->u.v);
 		break;
 	case isl_ast_expr_error:
 		break;
@@ -1270,6 +1655,11 @@ static __isl_give isl_printer *print_ast_node_isl(__isl_take isl_printer *p,
 			p = isl_printer_print_ast_node(p, node->u.f.body);
 		}
 		break;
+	case isl_ast_node_mark:
+		p = isl_printer_print_str(p, "mark: ");
+		p = isl_printer_print_id(p, node->u.m.mark);
+		p = isl_printer_print_str(p, "node: ");
+		p = isl_printer_print_ast_node(p, node->u.m.node);
 	case isl_ast_node_user:
 		p = isl_printer_print_ast_expr(p, node->u.e.expr);
 		break;
@@ -1290,7 +1680,7 @@ static __isl_give isl_printer *print_ast_node_isl(__isl_take isl_printer *p,
 	case isl_ast_node_block:
 		p = isl_printer_print_ast_node_list(p, node->u.b.children);
 		break;
-	default:
+	case isl_ast_node_error:
 		break;
 	}
 	p = isl_printer_print_str(p, ")");
@@ -1303,19 +1693,34 @@ static __isl_give isl_printer *print_ast_node_isl(__isl_take isl_printer *p,
  * Also if the node is a degenerate for then we will print it as
  * an assignment followed by the body of the for loop, so we need a block
  * as well.
+ * If the node is an if node with an else, then we print a block
+ * to avoid spurious dangling else warnings emitted by some compilers.
+ * If the node is a mark, then in principle, we would have to check
+ * the child of the mark node.  However, even if the child would not
+ * require us to print a block, for readability it is probably best
+ * to print a block anyway.
+ * If the ast_always_print_block option has been set, then we print a block.
  */
 static int need_block(__isl_keep isl_ast_node *node)
 {
+	isl_ctx *ctx;
+
 	if (node->type == isl_ast_node_block)
 		return 1;
 	if (node->type == isl_ast_node_for && node->u.f.degenerate)
 		return 1;
-	return 0;
+	if (node->type == isl_ast_node_if && node->u.i.else_node)
+		return 1;
+	if (node->type == isl_ast_node_mark)
+		return 1;
+
+	ctx = isl_ast_node_get_ctx(node);
+	return isl_options_get_ast_always_print_block(ctx);
 }
 
 static __isl_give isl_printer *print_ast_node_c(__isl_take isl_printer *p,
 	__isl_keep isl_ast_node *node,
-	__isl_keep isl_ast_print_options *options, int in_block);
+	__isl_keep isl_ast_print_options *options, int in_block, int in_list);
 static __isl_give isl_printer *print_if_c(__isl_take isl_printer *p,
 	__isl_keep isl_ast_node *node,
 	__isl_keep isl_ast_print_options *options, int new_line);
@@ -1355,7 +1760,7 @@ static __isl_give isl_printer *print_body_c(__isl_take isl_printer *p,
 	p = isl_printer_print_str(p, " {");
 	p = isl_printer_end_line(p);
 	p = isl_printer_indent(p, 2);
-	p = print_ast_node_c(p, node, options, 1);
+	p = print_ast_node_c(p, node, options, 1, 0);
 	p = isl_printer_indent(p, -2);
 	p = isl_printer_start_line(p);
 	p = isl_printer_print_str(p, "}");
@@ -1373,6 +1778,30 @@ static __isl_give isl_printer *print_body_c(__isl_take isl_printer *p,
 	return p;
 }
 
+/* Print the start of a compound statement.
+ */
+static __isl_give isl_printer *start_block(__isl_take isl_printer *p)
+{
+	p = isl_printer_start_line(p);
+	p = isl_printer_print_str(p, "{");
+	p = isl_printer_end_line(p);
+	p = isl_printer_indent(p, 2);
+
+	return p;
+}
+
+/* Print the end of a compound statement.
+ */
+static __isl_give isl_printer *end_block(__isl_take isl_printer *p)
+{
+	p = isl_printer_indent(p, -2);
+	p = isl_printer_start_line(p);
+	p = isl_printer_print_str(p, "}");
+	p = isl_printer_end_line(p);
+
+	return p;
+}
+
 /* Print the for node "node".
  *
  * If the for node is degenerate, it is printed as
@@ -1386,12 +1815,15 @@ static __isl_give isl_printer *print_body_c(__isl_take isl_printer *p,
  *		body
  *
  * "in_block" is set if we are currently inside a block.
- * We simply pass it along to print_ast_node_c in case of a degenerate
- * for loop.
+ * "in_list" is set if the current node is not alone in the block.
+ * If we are not in a block or if the current not is not alone in the block
+ * then we print a block around a degenerate for loop such that the variable
+ * declaration will not conflict with any potential other declaration
+ * of the same variable.
  */
 static __isl_give isl_printer *print_for_c(__isl_take isl_printer *p,
 	__isl_keep isl_ast_node *node,
-	__isl_keep isl_ast_print_options *options, int in_block)
+	__isl_keep isl_ast_print_options *options, int in_block, int in_list)
 {
 	isl_id *id;
 	const char *name;
@@ -1421,6 +1853,8 @@ static __isl_give isl_printer *print_for_c(__isl_take isl_printer *p,
 		id = isl_ast_expr_get_id(node->u.f.iterator);
 		name = isl_id_get_name(id);
 		isl_id_free(id);
+		if (!in_block || in_list)
+			p = start_block(p);
 		p = isl_printer_start_line(p);
 		p = isl_printer_print_str(p, type);
 		p = isl_printer_print_str(p, " ");
@@ -1429,7 +1863,9 @@ static __isl_give isl_printer *print_for_c(__isl_take isl_printer *p,
 		p = isl_printer_print_ast_expr(p, node->u.f.init);
 		p = isl_printer_print_str(p, ";");
 		p = isl_printer_end_line(p);
-		p = print_ast_node_c(p, node->u.f.body, options, in_block);
+		p = print_ast_node_c(p, node->u.f.body, options, 1, 0);
+		if (!in_block || in_list)
+			p = end_block(p);
 	}
 
 	return p;
@@ -1458,10 +1894,12 @@ static __isl_give isl_printer *print_if_c(__isl_take isl_printer *p,
  * If so, we do not print a block around the children of a block node.
  * We do this to avoid an extra block around the body of a degenerate
  * for node.
+ *
+ * "in_list" is set if the current node is not alone in the block.
  */
 static __isl_give isl_printer *print_ast_node_c(__isl_take isl_printer *p,
 	__isl_keep isl_ast_node *node,
-	__isl_keep isl_ast_print_options *options, int in_block)
+	__isl_keep isl_ast_print_options *options, int in_block, int in_list)
 {
 	switch (node->type) {
 	case isl_ast_node_for:
@@ -1469,25 +1907,24 @@ static __isl_give isl_printer *print_ast_node_c(__isl_take isl_printer *p,
 			return options->print_for(p,
 					isl_ast_print_options_copy(options),
 					node, options->print_for_user);
-		p = print_for_c(p, node, options, in_block);
+		p = print_for_c(p, node, options, in_block, in_list);
 		break;
 	case isl_ast_node_if:
 		p = print_if_c(p, node, options, 1);
 		break;
 	case isl_ast_node_block:
-		if (!in_block) {
-			p = isl_printer_start_line(p);
-			p = isl_printer_print_str(p, "{");
-			p = isl_printer_end_line(p);
-			p = isl_printer_indent(p, 2);
-		}
+		if (!in_block)
+			p = start_block(p);
 		p = isl_ast_node_list_print(node->u.b.children, p, options);
-		if (!in_block) {
-			p = isl_printer_indent(p, -2);
-			p = isl_printer_start_line(p);
-			p = isl_printer_print_str(p, "}");
-			p = isl_printer_end_line(p);
-		}
+		if (!in_block)
+			p = end_block(p);
+		break;
+	case isl_ast_node_mark:
+		p = isl_printer_start_line(p);
+		p = isl_printer_print_str(p, "// ");
+		p = isl_printer_print_str(p, isl_id_get_name(node->u.m.mark));
+		p = isl_printer_end_line(p);
+		p = print_ast_node_c(p, node->u.m.node, options, 0, in_list);
 		break;
 	case isl_ast_node_user:
 		if (options->print_user)
@@ -1515,7 +1952,7 @@ __isl_give isl_printer *isl_ast_node_for_print(__isl_keep isl_ast_node *node,
 	if (node->type != isl_ast_node_for)
 		isl_die(isl_ast_node_get_ctx(node), isl_error_invalid,
 			"not a for node", goto error);
-	p = print_for_c(p, node, options, 0);
+	p = print_for_c(p, node, options, 0, 0);
 	isl_ast_print_options_free(options);
 	return p;
 error:
@@ -1550,7 +1987,7 @@ __isl_give isl_printer *isl_ast_node_print(__isl_keep isl_ast_node *node,
 {
 	if (!options || !node)
 		goto error;
-	p = print_ast_node_c(p, node, options, 0);
+	p = print_ast_node_c(p, node, options, 0, 0);
 	isl_ast_print_options_free(options);
 	return p;
 error:
@@ -1600,7 +2037,7 @@ __isl_give isl_printer *isl_ast_node_list_print(
 		return isl_printer_free(p);
 
 	for (i = 0; i < list->n; ++i)
-		p = print_ast_node_c(p, list->p[i], options, 1);
+		p = print_ast_node_c(p, list->p[i], options, 1, 1);
 
 	return p;
 }
@@ -1671,6 +2108,9 @@ static int ast_node_required_macros(__isl_keep isl_ast_node *node, int macros)
 		macros = ast_node_list_required_macros(node->u.b.children,
 							macros);
 		break;
+	case isl_ast_node_mark:
+		macros = ast_node_required_macros(node->u.m.node, macros);
+		break;
 	case isl_ast_node_user:
 		macros = ast_expr_required_macros(node->u.e.expr, macros);
 		break;
@@ -1730,33 +2170,33 @@ __isl_give isl_printer *isl_ast_op_type_print_macro(
 /* Call "fn" for each type of operation that appears in "node"
  * and that requires a macro definition.
  */
-int isl_ast_node_foreach_ast_op_type(__isl_keep isl_ast_node *node,
-	int (*fn)(enum isl_ast_op_type type, void *user), void *user)
+isl_stat isl_ast_node_foreach_ast_op_type(__isl_keep isl_ast_node *node,
+	isl_stat (*fn)(enum isl_ast_op_type type, void *user), void *user)
 {
 	int macros;
 
 	if (!node)
-		return -1;
+		return isl_stat_error;
 
 	macros = ast_node_required_macros(node, 0);
 
 	if (macros & ISL_AST_MACRO_MIN && fn(isl_ast_op_min, user) < 0)
-		return -1;
+		return isl_stat_error;
 	if (macros & ISL_AST_MACRO_MAX && fn(isl_ast_op_max, user) < 0)
-		return -1;
+		return isl_stat_error;
 	if (macros & ISL_AST_MACRO_FLOORD && fn(isl_ast_op_fdiv_q, user) < 0)
-		return -1;
+		return isl_stat_error;
 
-	return 0;
+	return isl_stat_ok;
 }
 
-static int ast_op_type_print_macro(enum isl_ast_op_type type, void *user)
+static isl_stat ast_op_type_print_macro(enum isl_ast_op_type type, void *user)
 {
 	isl_printer **p = user;
 
 	*p = isl_ast_op_type_print_macro(type, *p);
 
-	return 0;
+	return isl_stat_ok;
 }
 
 /* Print macro definitions for all the macros used in the result
