@@ -1,13 +1,15 @@
 /*
  * Copyright 2012      Ecole Normale Superieure
+ * Copyright 2014      INRIA Rocquencourt
  *
  * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege,
  * Ecole Normale Superieure, 45 rue d’Ulm, 75230 Paris, France
+ * and Inria Paris - Rocquencourt, Domaine de Voluceau - Rocquencourt,
+ * B.P. 105 - 78153 Le Chesnay, France
  */
 
-#include <isl_list_private.h>
 #include <isl_ast_private.h>
 #include <isl_ast_build_expr.h>
 #include <isl_ast_build_private.h>
@@ -112,7 +114,11 @@ static int equal_independent_guards(__isl_keep isl_ast_graft_list *list,
 		return -1;
 
 	depth = isl_ast_build_get_depth(build);
-	skip = isl_set_involves_dims(graft_0->guard, isl_dim_set, depth, 1);
+	if (isl_set_dim(graft_0->guard, isl_dim_set) <= depth)
+		skip = 0;
+	else
+		skip = isl_set_involves_dims(graft_0->guard,
+						isl_dim_set, depth, 1);
 	if (skip < 0 || skip) {
 		isl_ast_graft_free(graft_0);
 		return skip < 0 ? -1 : 0;
@@ -136,26 +142,61 @@ static int equal_independent_guards(__isl_keep isl_ast_graft_list *list,
 	return equal;
 }
 
+/* Hoist "guard" out of the current level (given by "build").
+ *
+ * In particular, eliminate the dimension corresponding to the current depth.
+ */
+static __isl_give isl_set *hoist_guard(__isl_take isl_set *guard,
+	__isl_keep isl_ast_build *build)
+{
+	int depth;
+
+	depth = isl_ast_build_get_depth(build);
+	if (depth < isl_set_dim(guard, isl_dim_set)) {
+		guard = isl_set_remove_divs_involving_dims(guard,
+						isl_dim_set, depth, 1);
+		guard = isl_set_eliminate(guard, isl_dim_set, depth, 1);
+		guard = isl_set_compute_divs(guard);
+	}
+
+	return guard;
+}
+
 /* Extract a common guard from the grafts in "list" that can be hoisted
  * out of the current level.  If no such guard can be found, then return
  * a universal set.
  *
  * If all the grafts in the list have the same guard and if this guard
  * is independent of the current level, then it can be hoisted out.
+ * If there is only one graft in the list and if its guard
+ * depends on the current level, then we eliminate this level and
+ * return the result.
+ *
  * Otherwise, we return the unshifted simple hull of the guards.
+ * In order to be able to hoist as many constraints as possible,
+ * but at the same time avoid hoisting constraints that did not
+ * appear in the guards in the first place, we intersect the guards
+ * with all the information that is available (i.e., the domain
+ * from the build and the enforced constraints of the graft) and
+ * compute the unshifted hull of the result using only constraints
+ * from the original guards.
+ * In particular, intersecting the guards with other known information
+ * allows us to hoist guards that are only explicit is some of
+ * the grafts and implicit in the others.
  *
  * The special case for equal guards is needed in case those guards
  * are non-convex.  Taking the simple hull would remove information
  * and would not allow for these guards to be hoisted completely.
  */
-static __isl_give isl_set *extract_hoistable_guard(
+__isl_give isl_set *isl_ast_graft_list_extract_hoistable_guard(
 	__isl_keep isl_ast_graft_list *list, __isl_keep isl_ast_build *build)
 {
 	int i, n;
-	int depth;
-	isl_ast_graft *graft_0;
 	int equal;
+	isl_ctx *ctx;
 	isl_set *guard;
+	isl_set_list *set_list;
+	isl_basic_set *hull;
 
 	if (!list || !build)
 		return NULL;
@@ -168,43 +209,113 @@ static __isl_give isl_set *extract_hoistable_guard(
 	if (equal < 0)
 		return NULL;
 
-	graft_0 = isl_ast_graft_list_get_ast_graft(list, 0);
-	if (!graft_0)
-		return NULL;
-	guard = isl_set_copy(graft_0->guard);
-	isl_ast_graft_free(graft_0);
-	if (equal)
-		return guard;
+	if (equal || n == 1) {
+		isl_ast_graft *graft_0;
 
-	depth = isl_ast_build_get_depth(build);
-	if (depth < isl_set_dim(guard, isl_dim_set)) {
-		guard = isl_set_eliminate(guard, isl_dim_set, depth, 1);
-		guard = isl_set_compute_divs(guard);
+		graft_0 = isl_ast_graft_list_get_ast_graft(list, 0);
+		if (!graft_0)
+			return NULL;
+		guard = isl_set_copy(graft_0->guard);
+		if (!equal)
+			guard = hoist_guard(guard, build);
+		isl_ast_graft_free(graft_0);
+		return guard;
 	}
 
-	for (i = 1; i < n; ++i) {
+	ctx = isl_ast_build_get_ctx(build);
+	set_list = isl_set_list_alloc(ctx, n);
+	guard = isl_set_empty(isl_ast_build_get_space(build, 1));
+	for (i = 0; i < n; ++i) {
 		isl_ast_graft *graft;
-		isl_basic_set *hull;
-		int is_universe;
-
-		is_universe = isl_set_plain_is_universe(guard);
-		if (is_universe < 0)
-			guard = isl_set_free(guard);
-		if (is_universe)
-			break;
+		isl_basic_set *enforced;
+		isl_set *guard_i;
 
 		graft = isl_ast_graft_list_get_ast_graft(list, i);
-		if (!graft) {
-			guard = isl_set_free(guard);
-			break;
-		}
-		guard = isl_set_union(guard, isl_set_copy(graft->guard));
-		hull = isl_set_unshifted_simple_hull(guard);
-		guard = isl_set_from_basic_set(hull);
+		enforced = isl_ast_graft_get_enforced(graft);
+		guard_i = isl_set_copy(graft->guard);
 		isl_ast_graft_free(graft);
+		set_list = isl_set_list_add(set_list, isl_set_copy(guard_i));
+		guard_i = isl_set_intersect(guard_i,
+					    isl_set_from_basic_set(enforced));
+		guard_i = isl_set_intersect(guard_i,
+					    isl_ast_build_get_domain(build));
+		guard = isl_set_union(guard, guard_i);
+	}
+	hull = isl_set_unshifted_simple_hull_from_set_list(guard, set_list);
+	guard = isl_set_from_basic_set(hull);
+	return hoist_guard(guard, build);
+}
+
+/* Internal data structure used inside insert_if.
+ *
+ * list is the list of guarded nodes created by each call to insert_if.
+ * node is the original node that is guarded by insert_if.
+ * build is the build in which the AST is constructed.
+ */
+struct isl_insert_if_data {
+	isl_ast_node_list *list;
+	isl_ast_node *node;
+	isl_ast_build *build;
+};
+
+static isl_stat insert_if(__isl_take isl_basic_set *bset, void *user);
+
+/* Insert an if node around "node" testing the condition encoded
+ * in guard "guard".
+ *
+ * If the user does not want any disjunctions in the if conditions
+ * and if "guard" does involve a disjunction, then we make the different
+ * disjuncts disjoint and insert an if node corresponding to each disjunct
+ * around a copy of "node".  The result is then a block node containing
+ * this sequence of guarded copies of "node".
+ */
+static __isl_give isl_ast_node *ast_node_insert_if(
+	__isl_take isl_ast_node *node, __isl_take isl_set *guard,
+	__isl_keep isl_ast_build *build)
+{
+	struct isl_insert_if_data data;
+	isl_ctx *ctx;
+
+	ctx = isl_ast_build_get_ctx(build);
+	if (isl_options_get_ast_build_allow_or(ctx) ||
+	    isl_set_n_basic_set(guard) <= 1) {
+		isl_ast_node *if_node;
+		isl_ast_expr *expr;
+
+		expr = isl_ast_build_expr_from_set_internal(build, guard);
+
+		if_node = isl_ast_node_alloc_if(expr);
+		return isl_ast_node_if_set_then(if_node, node);
 	}
 
-	return guard;
+	guard = isl_set_make_disjoint(guard);
+
+	data.list = isl_ast_node_list_alloc(ctx, 0);
+	data.node = node;
+	data.build = build;
+	if (isl_set_foreach_basic_set(guard, &insert_if, &data) < 0)
+		data.list = isl_ast_node_list_free(data.list);
+
+	isl_set_free(guard);
+	isl_ast_node_free(data.node);
+	return isl_ast_node_alloc_block(data.list);
+}
+
+/* Insert an if node around a copy of "data->node" testing the condition
+ * encoded in guard "bset" and add the result to data->list.
+ */
+static isl_stat insert_if(__isl_take isl_basic_set *bset, void *user)
+{
+	struct isl_insert_if_data *data = user;
+	isl_ast_node *node;
+	isl_set *set;
+
+	set = isl_set_from_basic_set(bset);
+	node = isl_ast_node_copy(data->node);
+	node = ast_node_insert_if(node, set, data->build);
+	data->list = isl_ast_node_list_add(data->list, node);
+
+	return isl_stat_ok;
 }
 
 /* Insert an if node around graft->node testing the condition encoded
@@ -215,8 +326,6 @@ static __isl_give isl_ast_graft *insert_if_node(
 	__isl_keep isl_ast_build *build)
 {
 	int univ;
-	isl_ast_node *node;
-	isl_ast_expr *expr;
 
 	if (!graft)
 		goto error;
@@ -230,14 +339,9 @@ static __isl_give isl_ast_graft *insert_if_node(
 	}
 
 	build = isl_ast_build_copy(build);
-	build = isl_ast_build_set_enforced(build,
-					isl_ast_graft_get_enforced(graft));
-	expr = isl_ast_build_expr_from_set(build, guard);
+	graft->node = ast_node_insert_if(graft->node, guard, build);
 	isl_ast_build_free(build);
 
-	node = isl_ast_node_alloc_if(expr);
-	graft->node = isl_ast_node_if_set_then(node, graft->node);
-
 	if (!graft->node)
 		return isl_ast_graft_free(graft);
 
@@ -452,7 +556,7 @@ static __isl_give isl_ast_graft_list *insert_pending_guard_nodes(
 	allow_else = isl_options_get_ast_build_allow_else(ctx);
 
 	n_if = 0;
-	if (n > 0) {
+	if (n > 1) {
 		if_node = isl_alloc_array(ctx, struct isl_if_node, n - 1);
 		if (!if_node)
 			return isl_ast_graft_list_free(list);
@@ -549,6 +653,42 @@ static __isl_give isl_ast_graft_list *insert_pending_guard_nodes(
 	return res;
 }
 
+/* For each graft in "list",
+ * insert an if node around graft->node testing the condition encoded
+ * in graft->guard, assuming graft->guard involves any conditions.
+ * Subsequently remove the guards from the grafts.
+ */
+__isl_give isl_ast_graft_list *isl_ast_graft_list_insert_pending_guard_nodes(
+	__isl_take isl_ast_graft_list *list, __isl_keep isl_ast_build *build)
+{
+	int i, n;
+	isl_set *universe;
+
+	list = insert_pending_guard_nodes(list, build);
+	if (!list)
+		return NULL;
+
+	universe = isl_set_universe(isl_ast_build_get_space(build, 1));
+	n = isl_ast_graft_list_n_ast_graft(list);
+	for (i = 0; i < n; ++i) {
+		isl_ast_graft *graft;
+
+		graft = isl_ast_graft_list_get_ast_graft(list, i);
+		if (!graft)
+			break;
+		isl_set_free(graft->guard);
+		graft->guard = isl_set_copy(universe);
+		if (!graft->guard)
+			graft = isl_ast_graft_free(graft);
+		list = isl_ast_graft_list_set_ast_graft(list, i, graft);
+	}
+	isl_set_free(universe);
+	if (i < n)
+		return isl_ast_graft_list_free(list);
+
+	return list;
+}
+
 /* Collect the nodes contained in the grafts in "list" in a node list.
  */
 static __isl_give isl_ast_node_list *extract_node_list(
@@ -581,7 +721,7 @@ static __isl_give isl_ast_node_list *extract_node_list(
  *
  * We assume that the number of children is at least one.
  */
-static __isl_give isl_basic_set *extract_shared_enforced(
+__isl_give isl_basic_set *isl_ast_graft_list_extract_shared_enforced(
 	__isl_keep isl_ast_graft_list *list,
 	__isl_keep isl_ast_build *build)
 {
@@ -617,7 +757,10 @@ static __isl_give isl_basic_set *extract_shared_enforced(
 /* Record "guard" in "graft" so that it will be enforced somewhere
  * up the tree.  If the graft already has a guard, then it may be partially
  * redundant in combination with the new guard and in the context
- * of build->domain.  We therefore (re)compute the gist of the intersection.
+ * the generated constraints of "build".  In fact, the new guard
+ * may in itself have some redundant constraints.
+ * We therefore (re)compute the gist of the intersection
+ * and coalesce the result.
  */
 static __isl_give isl_ast_graft *store_guard(__isl_take isl_ast_graft *graft,
 	__isl_take isl_set *guard, __isl_keep isl_ast_build *build)
@@ -636,7 +779,9 @@ static __isl_give isl_ast_graft *store_guard(__isl_take isl_ast_graft *graft,
 	}
 
 	graft->guard = isl_set_intersect(graft->guard, guard);
-	graft->guard = isl_ast_build_compute_gist(build, graft->guard);
+	graft->guard = isl_set_gist(graft->guard,
+				    isl_ast_build_get_generated(build));
+	graft->guard = isl_set_coalesce(graft->guard);
 	if (!graft->guard)
 		return isl_ast_graft_free(graft);
 
@@ -676,14 +821,63 @@ static __isl_give isl_ast_graft_list *gist_guards(
 	return list;
 }
 
-/* Combine the grafts in the list into a single graft.
+/* For each graft in "list", replace its guard with the gist with
+ * respect to "context".
+ */
+__isl_give isl_ast_graft_list *isl_ast_graft_list_gist_guards(
+	__isl_take isl_ast_graft_list *list, __isl_take isl_set *context)
+{
+	list = gist_guards(list, context);
+	isl_set_free(context);
+
+	return list;
+}
+
+/* Allocate a graft in "build" based on the list of grafts in "sub_build".
+ * "guard" and "enforced" are the guard and enforced constraints
+ * of the allocated graft.  The guard is used to simplify the guards
+ * of the elements in "list".
  *
- * If "up" is set then the resulting graft will be used at an outer level.
+ * The node is initialized to either a block containing the nodes of "children"
+ * or, if there is only a single child, the node of that child.
+ * If the current level requires a for node, it should be inserted by
+ * a subsequent call to isl_ast_graft_insert_for.
+ */
+__isl_give isl_ast_graft *isl_ast_graft_alloc_from_children(
+	__isl_take isl_ast_graft_list *list, __isl_take isl_set *guard,
+	__isl_take isl_basic_set *enforced, __isl_keep isl_ast_build *build,
+	__isl_keep isl_ast_build *sub_build)
+{
+	isl_ast_build *guard_build;
+	isl_ast_node *node;
+	isl_ast_node_list *node_list;
+	isl_ast_graft *graft;
+
+	guard_build = isl_ast_build_copy(sub_build);
+	guard_build = isl_ast_build_replace_pending_by_guard(guard_build,
+						isl_set_copy(guard));
+	list = gist_guards(list, guard);
+	list = insert_pending_guard_nodes(list, guard_build);
+	isl_ast_build_free(guard_build);
+
+	node_list = extract_node_list(list);
+	node = isl_ast_node_from_ast_node_list(node_list);
+	isl_ast_graft_list_free(list);
+
+	graft = isl_ast_graft_alloc(node, build);
+	graft = store_guard(graft, guard, build);
+	graft = isl_ast_graft_enforce(graft, enforced);
+
+	return graft;
+}
+
+/* Combine the grafts in the list into a single graft.
  *
  * The guard is initialized to the shared guard of the list elements (if any),
  * provided it does not depend on the current dimension.
  * The guards in the elements are then simplified with respect to the
- * hoisted guard and materialized as if nodes around the contained AST nodes.
+ * hoisted guard and materialized as if nodes around the contained AST nodes
+ * in the context of "sub_build".
  *
  * The enforced set is initialized to the simple hull of the enforced sets
  * of the elements, provided the ast_build_exploit_nested_bounds option is set
@@ -693,37 +887,20 @@ static __isl_give isl_ast_graft_list *gist_guards(
  * or, if there is only a single element, the node of that element.
  */
 static __isl_give isl_ast_graft *ast_graft_list_fuse(
-	__isl_take isl_ast_graft_list *list,
-	__isl_keep isl_ast_build *build, int up)
+	__isl_take isl_ast_graft_list *list, __isl_keep isl_ast_build *build)
 {
-	isl_ctx *ctx;
-	isl_ast_node *node;
 	isl_ast_graft *graft;
-	isl_ast_node_list *node_list;
+	isl_basic_set *enforced;
 	isl_set *guard;
 
 	if (!list)
 		return NULL;
 
-	ctx = isl_ast_build_get_ctx(build);
-	guard = extract_hoistable_guard(list, build);
-	list = gist_guards(list, guard);
-	list = insert_pending_guard_nodes(list, build);
+	enforced = isl_ast_graft_list_extract_shared_enforced(list, build);
+	guard = isl_ast_graft_list_extract_hoistable_guard(list, build);
+	graft = isl_ast_graft_alloc_from_children(list, guard, enforced,
+						    build, build);
 
-	node_list = extract_node_list(list);
-	node = isl_ast_node_from_ast_node_list(node_list);
-
-	graft = isl_ast_graft_alloc(node, build);
-
-	if (!up || isl_options_get_ast_build_exploit_nested_bounds(ctx)) {
-		isl_basic_set *enforced;
-		enforced = extract_shared_enforced(list, build);
-		graft = isl_ast_graft_enforce(graft, enforced);
-	}
-
-	graft = store_guard(graft, guard, build);
-
-	isl_ast_graft_list_free(list);
 	return graft;
 }
 
@@ -741,7 +918,7 @@ __isl_give isl_ast_graft_list *isl_ast_graft_list_fuse(
 		return NULL;
 	if (isl_ast_graft_list_n_ast_graft(list) <= 1)
 		return list;
-	graft = ast_graft_list_fuse(list, build, 0);
+	graft = ast_graft_list_fuse(list, build);
 	return isl_ast_graft_list_from_ast_graft(graft);
 }
 
@@ -761,22 +938,7 @@ static __isl_give isl_ast_graft *isl_ast_graft_fuse(
 	list = isl_ast_graft_list_add(list, graft1);
 	list = isl_ast_graft_list_add(list, graft2);
 
-	return ast_graft_list_fuse(list, build, 0);
-}
-
-/* Allocate a graft for the current level based on the list of grafts
- * of the inner level.
- *
- * The node is initialized to either a block containing the nodes of "children"
- * or, if there is only a single child, the node of that child.
- * If the current level requires a for node, it should be inserted by
- * a subsequent call to isl_ast_graft_insert_for.
- */
-__isl_give isl_ast_graft *isl_ast_graft_alloc_level(
-	__isl_take isl_ast_graft_list *children,
-	__isl_keep isl_ast_build *build)
-{
-	return ast_graft_list_fuse(children, build, 1);
+	return ast_graft_list_fuse(list, build);
 }
 
 /* Insert a for node enclosing the current graft->node.
@@ -798,6 +960,25 @@ error:
 	return NULL;
 }
 
+/* Insert a mark governing the current graft->node.
+ */
+__isl_give isl_ast_graft *isl_ast_graft_insert_mark(
+	__isl_take isl_ast_graft *graft, __isl_take isl_id *mark)
+{
+	if (!graft)
+		goto error;
+
+	graft->node = isl_ast_node_alloc_mark(mark, graft->node);
+	if (!graft->node)
+		return isl_ast_graft_free(graft);
+
+	return graft;
+error:
+	isl_id_free(mark);
+	isl_ast_graft_free(graft);
+	return NULL;
+}
+
 /* Represent the graft list as an AST node.
  * This operation drops the information about guards in the grafts, so
  * if there are any pending guards, then they are materialized as if nodes.
@@ -866,30 +1047,12 @@ __isl_give isl_set *isl_ast_graft_get_guard(__isl_keep isl_ast_graft *graft)
 }
 
 /* Record that "guard" needs to be inserted in "graft".
- *
- * We first simplify the guard in the context of the enforced set and
- * then we store the guard in case we may be able
- * to hoist it to higher levels and/or combine it with those of other grafts.
  */
 __isl_give isl_ast_graft *isl_ast_graft_add_guard(
 	__isl_take isl_ast_graft *graft,
 	__isl_take isl_set *guard, __isl_keep isl_ast_build *build)
 {
-	isl_basic_set *enforced;
-
-	if (!graft || !build)
-		goto error;
-
-	enforced = isl_basic_set_copy(graft->enforced);
-	guard = isl_set_gist(guard, isl_set_from_basic_set(enforced));
-
-	graft = store_guard(graft, guard, build);
-
-	return graft;
-error:
-	isl_set_free(guard);
-	isl_ast_graft_free(graft);
-	return NULL;
+	return store_guard(graft, guard, build);
 }
 
 /* Reformulate the "graft", which was generated in the context
@@ -998,27 +1161,18 @@ __isl_give isl_ast_graft_list *isl_ast_graft_list_preimage_multi_aff(
 
 /* Compare two grafts based on their guards.
  */
-static int cmp_graft(const void *a, const void *b)
+static int cmp_graft(__isl_keep isl_ast_graft *a, __isl_keep isl_ast_graft *b,
+	void *user)
 {
-	isl_ast_graft * const *g1 = a;
-	isl_ast_graft * const *g2 = b;
-
-	return isl_set_plain_cmp((*g1)->guard, (*g2)->guard);
+	return isl_set_plain_cmp(a->guard, b->guard);
 }
 
 /* Order the elements in "list" based on their guards.
  */
-__isl_give isl_ast_graft_list *isl_ast_graft_list_sort(
+__isl_give isl_ast_graft_list *isl_ast_graft_list_sort_guard(
 	__isl_take isl_ast_graft_list *list)
 {
-	if (!list)
-		return NULL;
-	if (list->n <= 1)
-		return list;
-
-	qsort(list->p, list->n, sizeof(list->p[0]), &cmp_graft);
-
-	return list;
+	return isl_ast_graft_list_sort(list, &cmp_graft, NULL);
 }
 
 /* Merge the given two lists into a single list of grafts,
