@@ -1,3 +1,16 @@
+/*
+ * Copyright 2010-2011 INRIA Saclay
+ * Copyright 2011      Sven Verdoolaege
+ * Copyright 2012-2014 Ecole Normale Superieure
+ *
+ * Use of this software is governed by the MIT license
+ *
+ * Written by Sven Verdoolaege, INRIA Saclay - Ile-de-France,
+ * Parc Club Orsay Universite, ZAC des vignes, 4 rue Jacques Monod,
+ * 91893 Orsay, France
+ * and Ecole Normale Superieure, 45 rue d’Ulm, 75230 Paris, France
+ */
+
 #include <isl/aff.h>
 #include <isl_val_private.h>
 
@@ -157,7 +170,7 @@ __isl_give PW *FN(PW,copy)(__isl_keep PW *pw)
 	return pw;
 }
 
-void *FN(PW,free)(__isl_take PW *pw)
+__isl_null PW *FN(PW,free)(__isl_take PW *pw)
 {
 	int i;
 
@@ -182,9 +195,10 @@ const char *FN(PW,get_dim_name)(__isl_keep PW *pw, enum isl_dim_type type,
 	return pw ? isl_space_get_dim_name(pw->dim, type, pos) : NULL;
 }
 
-int FN(PW,has_dim_id)(__isl_keep PW *pw, enum isl_dim_type type, unsigned pos)
+isl_bool FN(PW,has_dim_id)(__isl_keep PW *pw, enum isl_dim_type type,
+	unsigned pos)
 {
-	return pw ? isl_space_has_dim_id(pw->dim, type, pos) : -1;
+	return pw ? isl_space_has_dim_id(pw->dim, type, pos) : isl_bool_error;
 }
 
 __isl_give isl_id *FN(PW,get_dim_id)(__isl_keep PW *pw, enum isl_dim_type type,
@@ -193,9 +207,9 @@ __isl_give isl_id *FN(PW,get_dim_id)(__isl_keep PW *pw, enum isl_dim_type type,
 	return pw ? isl_space_get_dim_id(pw->dim, type, pos) : NULL;
 }
 
-int FN(PW,has_tuple_name)(__isl_keep PW *pw, enum isl_dim_type type)
+isl_bool FN(PW,has_tuple_name)(__isl_keep PW *pw, enum isl_dim_type type)
 {
-	return pw ? isl_space_has_tuple_name(pw->dim, type) : -1;
+	return pw ? isl_space_has_tuple_name(pw->dim, type) : isl_bool_error;
 }
 
 const char *FN(PW,get_tuple_name)(__isl_keep PW *pw, enum isl_dim_type type)
@@ -203,9 +217,9 @@ const char *FN(PW,get_tuple_name)(__isl_keep PW *pw, enum isl_dim_type type)
 	return pw ? isl_space_get_tuple_name(pw->dim, type) : NULL;
 }
 
-int FN(PW,has_tuple_id)(__isl_keep PW *pw, enum isl_dim_type type)
+isl_bool FN(PW,has_tuple_id)(__isl_keep PW *pw, enum isl_dim_type type)
 {
-	return pw ? isl_space_has_tuple_id(pw->dim, type) : -1;
+	return pw ? isl_space_has_tuple_id(pw->dim, type) : isl_bool_error;
 }
 
 __isl_give isl_id *FN(PW,get_tuple_id)(__isl_keep PW *pw, enum isl_dim_type type)
@@ -213,10 +227,10 @@ __isl_give isl_id *FN(PW,get_tuple_id)(__isl_keep PW *pw, enum isl_dim_type type
 	return pw ? isl_space_get_tuple_id(pw->dim, type) : NULL;
 }
 
-int FN(PW,IS_ZERO)(__isl_keep PW *pw)
+isl_bool FN(PW,IS_ZERO)(__isl_keep PW *pw)
 {
 	if (!pw)
-		return -1;
+		return isl_bool_error;
 
 	return pw->n == 0;
 }
@@ -631,7 +645,9 @@ __isl_give PW *FN(PW,neg)(__isl_take PW *pw)
 
 	return pw;
 }
+#endif
 
+#ifndef NO_SUB
 __isl_give PW *FN(PW,sub)(__isl_take PW *pw1, __isl_take PW *pw2)
 {
 	return FN(PW,add)(pw1, FN(PW,neg)(pw2));
@@ -639,14 +655,13 @@ __isl_give PW *FN(PW,sub)(__isl_take PW *pw1, __isl_take PW *pw2)
 #endif
 
 #ifndef NO_EVAL
-__isl_give isl_qpolynomial *FN(PW,eval)(__isl_take PW *pw,
-	__isl_take isl_point *pnt)
+__isl_give isl_val *FN(PW,eval)(__isl_take PW *pw, __isl_take isl_point *pnt)
 {
 	int i;
 	int found = 0;
 	isl_ctx *ctx;
 	isl_space *pnt_dim = NULL;
-	isl_qpolynomial *qp;
+	isl_val *v;
 
 	if (!pw || !pnt)
 		goto error;
@@ -663,14 +678,14 @@ __isl_give isl_qpolynomial *FN(PW,eval)(__isl_take PW *pw,
 			break;
 	}
 	if (found)
-		qp = FN(EL,eval)(FN(EL,copy)(pw->p[i].FIELD),
+		v = FN(EL,eval)(FN(EL,copy)(pw->p[i].FIELD),
 					    isl_point_copy(pnt));
 	else
-		qp = isl_qpolynomial_zero_on_domain(FN(PW,get_domain_space)(pw));
+		v = isl_val_zero(ctx);
 	FN(PW,free)(pw);
 	isl_space_free(pnt_dim);
 	isl_point_free(pnt);
-	return qp;
+	return v;
 error:
 	FN(PW,free)(pw);
 	isl_space_free(pnt_dim);
@@ -679,6 +694,13 @@ error:
 }
 #endif
 
+/* Return the parameter domain of "pw".
+ */
+__isl_give isl_set *FN(PW,params)(__isl_take PW *pw)
+{
+	return isl_set_params(FN(PW,domain)(pw));
+}
+
 __isl_give isl_set *FN(PW,domain)(__isl_take PW *pw)
 {
 	int i;
@@ -727,11 +749,65 @@ static int FN(PW,exploit_equalities_and_remove_if_empty)(__isl_keep PW *pw,
 	return 0;
 }
 
+/* Convert a piecewise expression defined over a parameter domain
+ * into one that is defined over a zero-dimensional set.
+ */
+__isl_give PW *FN(PW,from_range)(__isl_take PW *pw)
+{
+	isl_space *space;
+
+	if (!pw)
+		return NULL;
+	if (!isl_space_is_set(pw->dim))
+		isl_die(FN(PW,get_ctx)(pw), isl_error_invalid,
+			"not living in a set space", return FN(PW,free)(pw));
+
+	space = FN(PW,get_space)(pw);
+	space = isl_space_from_range(space);
+	pw = FN(PW,reset_space)(pw, space);
+
+	return pw;
+}
+
+/* Fix the value of the given parameter or domain dimension of "pw"
+ * to be equal to "value".
+ */
+__isl_give PW *FN(PW,fix_si)(__isl_take PW *pw, enum isl_dim_type type,
+	unsigned pos, int value)
+{
+	int i;
+
+	if (!pw)
+		return NULL;
+
+	if (type == isl_dim_out)
+		isl_die(FN(PW,get_ctx)(pw), isl_error_invalid,
+			"cannot fix output dimension", return FN(PW,free)(pw));
+
+	if (pw->n == 0)
+		return pw;
+
+	if (type == isl_dim_in)
+		type = isl_dim_set;
+
+	pw = FN(PW,cow)(pw);
+	if (!pw)
+		return FN(PW,free)(pw);
+
+	for (i = pw->n - 1; i >= 0; --i) {
+		pw->p[i].set = isl_set_fix_si(pw->p[i].set, type, pos, value);
+		if (FN(PW,exploit_equalities_and_remove_if_empty)(pw, i) < 0)
+			return FN(PW,free)(pw);
+	}
+
+	return pw;
+}
+
 /* Restrict the domain of "pw" by combining each cell
  * with "set" through a call to "fn", where "fn" may be
- * isl_set_intersect or isl_set_intersect_params.
+ * isl_set_intersect, isl_set_intersect_params or isl_set_subtract.
  */
-static __isl_give PW *FN(PW,intersect_aligned)(__isl_take PW *pw,
+static __isl_give PW *FN(PW,restrict_domain_aligned)(__isl_take PW *pw,
 	__isl_take isl_set *set,
 	__isl_give isl_set *(*fn)(__isl_take isl_set *set1,
 				    __isl_take isl_set *set2))
@@ -767,7 +843,7 @@ error:
 static __isl_give PW *FN(PW,intersect_domain_aligned)(__isl_take PW *pw,
 	__isl_take isl_set *set)
 {
-	return FN(PW,intersect_aligned)(pw, set, &isl_set_intersect);
+	return FN(PW,restrict_domain_aligned)(pw, set, &isl_set_intersect);
 }
 
 __isl_give PW *FN(PW,intersect_domain)(__isl_take PW *pw,
@@ -780,7 +856,8 @@ __isl_give PW *FN(PW,intersect_domain)(__isl_take PW *pw,
 static __isl_give PW *FN(PW,intersect_params_aligned)(__isl_take PW *pw,
 	__isl_take isl_set *set)
 {
-	return FN(PW,intersect_aligned)(pw, set, &isl_set_intersect_params);
+	return FN(PW,restrict_domain_aligned)(pw, set,
+					&isl_set_intersect_params);
 }
 
 /* Intersect the domain of "pw" with the parameter domain "context".
@@ -792,6 +869,65 @@ __isl_give PW *FN(PW,intersect_params)(__isl_take PW *pw,
 					&FN(PW,intersect_params_aligned));
 }
 
+/* Subtract "domain' from the domain of "pw", assuming their
+ * parameters have been aligned.
+ */
+static __isl_give PW *FN(PW,subtract_domain_aligned)(__isl_take PW *pw,
+	__isl_take isl_set *domain)
+{
+	return FN(PW,restrict_domain_aligned)(pw, domain, &isl_set_subtract);
+}
+
+/* Subtract "domain' from the domain of "pw".
+ */
+__isl_give PW *FN(PW,subtract_domain)(__isl_take PW *pw,
+	__isl_take isl_set *domain)
+{
+	return FN(PW,align_params_pw_set_and)(pw, domain,
+					&FN(PW,subtract_domain_aligned));
+}
+
+/* Compute the gist of "pw" with respect to the domain constraints
+ * of "context" for the case where the domain of the last element
+ * of "pw" is equal to "context".
+ * Call "fn_el" to compute the gist of this element, replace
+ * its domain by the universe and drop all other elements
+ * as their domains are necessarily disjoint from "context".
+ */
+static __isl_give PW *FN(PW,gist_last)(__isl_take PW *pw,
+	__isl_take isl_set *context,
+	__isl_give EL *(*fn_el)(__isl_take EL *el, __isl_take isl_set *set))
+{
+	int i;
+	isl_space *space;
+
+	for (i = 0; i < pw->n - 1; ++i) {
+		isl_set_free(pw->p[i].set);
+		FN(EL,free)(pw->p[i].FIELD);
+	}
+	pw->p[0].FIELD = pw->p[pw->n - 1].FIELD;
+	pw->p[0].set = pw->p[pw->n - 1].set;
+	pw->n = 1;
+
+	space = isl_set_get_space(context);
+	pw->p[0].FIELD = fn_el(pw->p[0].FIELD, context);
+	context = isl_set_universe(space);
+	isl_set_free(pw->p[0].set);
+	pw->p[0].set = context;
+
+	if (!pw->p[0].FIELD || !pw->p[0].set)
+		return FN(PW,free)(pw);
+
+	return pw;
+}
+
+/* Compute the gist of "pw" with respect to the domain constraints
+ * of "context".  Call "fn_el" to compute the gist of the elements
+ * and "fn_dom" to compute the gist of the domains.
+ *
+ * If the piecewise expression is empty or the context is the universe,
+ * then nothing can be simplified.
+ */
 static __isl_give PW *FN(PW,gist_aligned)(__isl_take PW *pw,
 	__isl_take isl_set *context,
 	__isl_give EL *(*fn_el)(__isl_take EL *el,
@@ -800,6 +936,7 @@ static __isl_give PW *FN(PW,gist_aligned)(__isl_take PW *pw,
 				    __isl_take isl_basic_set *bset))
 {
 	int i;
+	int is_universe;
 	isl_basic_set *hull = NULL;
 
 	if (!pw || !context)
@@ -810,29 +947,57 @@ static __isl_give PW *FN(PW,gist_aligned)(__isl_take PW *pw,
 		return pw;
 	}
 
+	is_universe = isl_set_plain_is_universe(context);
+	if (is_universe < 0)
+		goto error;
+	if (is_universe) {
+		isl_set_free(context);
+		return pw;
+	}
+
 	if (!isl_space_match(pw->dim, isl_dim_param,
 				context->dim, isl_dim_param)) {
 		pw = FN(PW,align_params)(pw, isl_set_get_space(context));
 		context = isl_set_align_params(context, FN(PW,get_space)(pw));
 	}
 
-	context = isl_set_compute_divs(context);
-	hull = isl_set_simple_hull(isl_set_copy(context));
-
 	pw = FN(PW,cow)(pw);
 	if (!pw)
 		goto error;
 
+	if (pw->n == 1) {
+		int equal;
+
+		equal = isl_set_plain_is_equal(pw->p[0].set, context);
+		if (equal < 0)
+			goto error;
+		if (equal)
+			return FN(PW,gist_last)(pw, context, fn_el);
+	}
+
+	context = isl_set_compute_divs(context);
+	hull = isl_set_simple_hull(isl_set_copy(context));
+
 	for (i = pw->n - 1; i >= 0; --i) {
 		isl_set *set_i;
 		int empty;
 
+		if (i == pw->n - 1) {
+			int equal;
+			equal = isl_set_plain_is_equal(pw->p[i].set, context);
+			if (equal < 0)
+				goto error;
+			if (equal) {
+				isl_basic_set_free(hull);
+				return FN(PW,gist_last)(pw, context, fn_el);
+			}
+		}
 		set_i = isl_set_intersect(isl_set_copy(pw->p[i].set),
 						 isl_set_copy(context));
 		empty = isl_set_plain_is_empty(set_i);
 		pw->p[i].FIELD = fn_el(pw->p[i].FIELD, set_i);
 		pw->p[i].set = fn_dom(pw->p[i].set, isl_basic_set_copy(hull));
-		if (!pw->p[i].FIELD || !pw->p[i].set)
+		if (empty < 0 || !pw->p[i].FIELD || !pw->p[i].set)
 			goto error;
 		if (empty) {
 			isl_set_free(pw->p[i].set);
@@ -922,21 +1087,21 @@ isl_ctx *FN(PW,get_ctx)(__isl_keep PW *pw)
 }
 
 #ifndef NO_INVOLVES_DIMS
-int FN(PW,involves_dims)(__isl_keep PW *pw, enum isl_dim_type type,
+isl_bool FN(PW,involves_dims)(__isl_keep PW *pw, enum isl_dim_type type,
 	unsigned first, unsigned n)
 {
 	int i;
 	enum isl_dim_type set_type;
 
 	if (!pw)
-		return -1;
+		return isl_bool_error;
 	if (pw->n == 0 || n == 0)
-		return 0;
+		return isl_bool_false;
 
 	set_type = type == isl_dim_in ? isl_dim_set : type;
 
 	for (i = 0; i < pw->n; ++i) {
-		int involves = FN(EL,involves_dims)(pw->p[i].FIELD,
+		isl_bool involves = FN(EL,involves_dims)(pw->p[i].FIELD,
 							type, first, n);
 		if (involves < 0 || involves)
 			return involves;
@@ -945,7 +1110,7 @@ int FN(PW,involves_dims)(__isl_keep PW *pw, enum isl_dim_type type,
 		if (involves < 0 || involves)
 			return involves;
 	}
-	return 0;
+	return isl_bool_false;
 }
 #endif
 
@@ -1199,42 +1364,42 @@ error:
  * In the worst case, the domain is scanned completely,
  * so the domain is assumed to be bounded.
  */
-__isl_give isl_qpolynomial *FN(PW,opt)(__isl_take PW *pw, int max)
+__isl_give isl_val *FN(PW,opt)(__isl_take PW *pw, int max)
 {
 	int i;
-	isl_qpolynomial *opt;
+	isl_val *opt;
 
 	if (!pw)
 		return NULL;
 
 	if (pw->n == 0) {
-		isl_space *dim = isl_space_copy(pw->dim);
+		opt = isl_val_zero(FN(PW,get_ctx)(pw));
 		FN(PW,free)(pw);
-		return isl_qpolynomial_zero_on_domain(isl_space_domain(dim));
+		return opt;
 	}
 
 	opt = FN(EL,opt_on_domain)(FN(EL,copy)(pw->p[0].FIELD),
 					isl_set_copy(pw->p[0].set), max);
 	for (i = 1; i < pw->n; ++i) {
-		isl_qpolynomial *opt_i;
+		isl_val *opt_i;
 		opt_i = FN(EL,opt_on_domain)(FN(EL,copy)(pw->p[i].FIELD),
 						isl_set_copy(pw->p[i].set), max);
 		if (max)
-			opt = isl_qpolynomial_max_cst(opt, opt_i);
+			opt = isl_val_max(opt, opt_i);
 		else
-			opt = isl_qpolynomial_min_cst(opt, opt_i);
+			opt = isl_val_min(opt, opt_i);
 	}
 
 	FN(PW,free)(pw);
 	return opt;
 }
 
-__isl_give isl_qpolynomial *FN(PW,max)(__isl_take PW *pw)
+__isl_give isl_val *FN(PW,max)(__isl_take PW *pw)
 {
 	return FN(PW,opt)(pw, 1);
 }
 
-__isl_give isl_qpolynomial *FN(PW,min)(__isl_take PW *pw)
+__isl_give isl_val *FN(PW,min)(__isl_take PW *pw)
 {
 	return FN(PW,opt)(pw, 0);
 }
@@ -1250,6 +1415,18 @@ __isl_give isl_space *FN(PW,get_domain_space)(__isl_keep PW *pw)
 	return pw ? isl_space_domain(isl_space_copy(pw->dim)) : NULL;
 }
 
+/* Return the position of the dimension of the given type and name
+ * in "pw".
+ * Return -1 if no such dimension can be found.
+ */
+int FN(PW,find_dim_by_name)(__isl_keep PW *pw,
+	enum isl_dim_type type, const char *name)
+{
+	if (!pw)
+		return -1;
+	return isl_space_find_dim_by_name(pw->dim, type, name);
+}
+
 #ifndef NO_RESET_DIM
 /* Reset the space of "pw".  Since we don't know if the elements
  * represent the spaces themselves or their domains, we pass along
@@ -1306,18 +1483,42 @@ __isl_give PW *FN(PW,reset_space)(__isl_take PW *pw, __isl_take isl_space *dim)
 	return FN(PW,reset_space_and_domain)(pw, dim, domain);
 }
 
-__isl_give PW *FN(PW,set_tuple_id)(__isl_keep PW *pw, enum isl_dim_type type,
+__isl_give PW *FN(PW,set_tuple_id)(__isl_take PW *pw, enum isl_dim_type type,
 	__isl_take isl_id *id)
 {
 	isl_space *space;
 
 	pw = FN(PW,cow)(pw);
 	if (!pw)
-		return isl_id_free(id);
+		goto error;
 
 	space = FN(PW,get_space)(pw);
 	space = isl_space_set_tuple_id(space, type, id);
 
+	return FN(PW,reset_space)(pw, space);
+error:
+	isl_id_free(id);
+	return FN(PW,free)(pw);
+}
+
+/* Drop the id on the specified tuple.
+ */
+__isl_give PW *FN(PW,reset_tuple_id)(__isl_take PW *pw, enum isl_dim_type type)
+{
+	isl_space *space;
+
+	if (!pw)
+		return NULL;
+	if (!FN(PW,has_tuple_id)(pw, type))
+		return pw;
+
+	pw = FN(PW,cow)(pw);
+	if (!pw)
+		return NULL;
+
+	space = FN(PW,get_space)(pw);
+	space = isl_space_reset_tuple_id(space, type);
+
 	return FN(PW,reset_space)(pw, space);
 }
 
@@ -1326,12 +1527,28 @@ __isl_give PW *FN(PW,set_dim_id)(__isl_take PW *pw,
 {
 	pw = FN(PW,cow)(pw);
 	if (!pw)
-		return isl_id_free(id);
+		goto error;
 	pw->dim = isl_space_set_dim_id(pw->dim, type, pos, id);
 	return FN(PW,reset_space)(pw, isl_space_copy(pw->dim));
+error:
+	isl_id_free(id);
+	return FN(PW,free)(pw);
 }
 #endif
 
+/* Reset the user pointer on all identifiers of parameters and tuples
+ * of the space of "pw".
+ */
+__isl_give PW *FN(PW,reset_user)(__isl_take PW *pw)
+{
+	isl_space *space;
+
+	space = FN(PW,get_space)(pw);
+	space = isl_space_reset_user(space);
+
+	return FN(PW,reset_space)(pw, space);
+}
+
 int FN(PW,has_equal_space)(__isl_keep PW *pw1, __isl_keep PW *pw2)
 {
 	if (!pw1 || !pw2)
@@ -1387,21 +1604,21 @@ int FN(PW,n_piece)(__isl_keep PW *pw)
 	return pw ? pw->n : 0;
 }
 
-int FN(PW,foreach_piece)(__isl_keep PW *pw,
-	int (*fn)(__isl_take isl_set *set, __isl_take EL *el, void *user),
+isl_stat FN(PW,foreach_piece)(__isl_keep PW *pw,
+	isl_stat (*fn)(__isl_take isl_set *set, __isl_take EL *el, void *user),
 	void *user)
 {
 	int i;
 
 	if (!pw)
-		return -1;
+		return isl_stat_error;
 
 	for (i = 0; i < pw->n; ++i)
 		if (fn(isl_set_copy(pw->p[i].set),
 				FN(EL,copy)(pw->p[i].FIELD), user) < 0)
-			return -1;
+			return isl_stat_error;
 
-	return 0;
+	return isl_stat_ok;
 }
 
 #ifndef NO_LIFT
@@ -1419,9 +1636,10 @@ static int any_divs(__isl_keep isl_set *set)
 	return 0;
 }
 
-static int foreach_lifted_subset(__isl_take isl_set *set, __isl_take EL *el,
-	int (*fn)(__isl_take isl_set *set, __isl_take EL *el,
-		    void *user), void *user)
+static isl_stat foreach_lifted_subset(__isl_take isl_set *set,
+	__isl_take EL *el,
+	isl_stat (*fn)(__isl_take isl_set *set, __isl_take EL *el,
+		void *user), void *user)
 {
 	int i;
 
@@ -1445,21 +1663,21 @@ static int foreach_lifted_subset(__isl_take isl_set *set, __isl_take EL *el,
 	isl_set_free(set);
 	FN(EL,free)(el);
 
-	return 0;
+	return isl_stat_ok;
 error:
 	isl_set_free(set);
 	FN(EL,free)(el);
-	return -1;
+	return isl_stat_error;
 }
 
-int FN(PW,foreach_lifted_piece)(__isl_keep PW *pw,
-	int (*fn)(__isl_take isl_set *set, __isl_take EL *el,
+isl_stat FN(PW,foreach_lifted_piece)(__isl_keep PW *pw,
+	isl_stat (*fn)(__isl_take isl_set *set, __isl_take EL *el,
 		    void *user), void *user)
 {
 	int i;
 
 	if (!pw)
-		return -1;
+		return isl_stat_error;
 
 	for (i = 0; i < pw->n; ++i) {
 		isl_set *set;
@@ -1469,14 +1687,14 @@ int FN(PW,foreach_lifted_piece)(__isl_keep PW *pw,
 		el = FN(EL,copy)(pw->p[i].FIELD);
 		if (!any_divs(set)) {
 			if (fn(set, el, user) < 0)
-				return -1;
+				return isl_stat_error;
 			continue;
 		}
 		if (foreach_lifted_subset(set, el, fn, user) < 0)
-			return -1;
+			return isl_stat_error;
 	}
 
-	return 0;
+	return isl_stat_ok;
 }
 #endif
 
@@ -1613,6 +1831,54 @@ error:
 	return NULL;
 }
 
+/* Divide the pieces of "pw" by "v" and return the result.
+ */
+__isl_give PW *FN(PW,scale_down_val)(__isl_take PW *pw, __isl_take isl_val *v)
+{
+	int i;
+
+	if (!pw || !v)
+		goto error;
+
+	if (isl_val_is_one(v)) {
+		isl_val_free(v);
+		return pw;
+	}
+
+	if (!isl_val_is_rat(v))
+		isl_die(isl_val_get_ctx(v), isl_error_invalid,
+			"expecting rational factor", goto error);
+	if (isl_val_is_zero(v))
+		isl_die(isl_val_get_ctx(v), isl_error_invalid,
+			"cannot scale down by zero", goto error);
+
+	if (pw->n == 0) {
+		isl_val_free(v);
+		return pw;
+	}
+	pw = FN(PW,cow)(pw);
+	if (!pw)
+		goto error;
+
+#ifdef HAS_TYPE
+	if (isl_val_is_neg(v))
+		pw->type = isl_fold_type_negate(pw->type);
+#endif
+	for (i = 0; i < pw->n; ++i) {
+		pw->p[i].FIELD = FN(EL,scale_down_val)(pw->p[i].FIELD,
+						    isl_val_copy(v));
+		if (!pw->p[i].FIELD)
+			goto error;
+	}
+
+	isl_val_free(v);
+	return pw;
+error:
+	isl_val_free(v);
+	FN(PW,free)(pw);
+	return NULL;
+}
+
 __isl_give PW *FN(PW,scale)(__isl_take PW *pw, isl_int v)
 {
 	return FN(PW,mul_isl_int)(pw, v);
@@ -1670,18 +1936,18 @@ __isl_give PW *FN(PW,normalize)(__isl_take PW *pw)
  * That is, do they have obviously identical cells and obviously identical
  * elements on each cell?
  */
-int FN(PW,plain_is_equal)(__isl_keep PW *pw1, __isl_keep PW *pw2)
+isl_bool FN(PW,plain_is_equal)(__isl_keep PW *pw1, __isl_keep PW *pw2)
 {
 	int i;
-	int equal;
+	isl_bool equal;
 
 	if (!pw1 || !pw2)
-		return -1;
+		return isl_bool_error;
 
 	if (pw1 == pw2)
-		return 1;
+		return isl_bool_true;
 	if (!isl_space_is_equal(pw1->dim, pw2->dim))
-		return 0;
+		return isl_bool_false;
 
 	pw1 = FN(PW,copy)(pw1);
 	pw2 = FN(PW,copy)(pw2);
@@ -1708,7 +1974,7 @@ int FN(PW,plain_is_equal)(__isl_keep PW *pw1, __isl_keep PW *pw2)
 error:
 	FN(PW,free)(pw1);
 	FN(PW,free)(pw2);
-	return -1;
+	return isl_bool_error;
 }
 
 #ifndef NO_PULLBACK
@@ -1829,8 +2095,11 @@ static __isl_give PW *FN(PW,pullback_pw_multi_aff_aligned)(__isl_take PW *pw,
 		goto error;
 
 	if (pma->n == 0) {
+		isl_space *space;
+		space = isl_space_join(isl_pw_multi_aff_get_space(pma),
+					FN(PW,get_space)(pw));
 		isl_pw_multi_aff_free(pma);
-		res = FN(PW,empty)(FN(PW,get_space)(pw));
+		res = FN(PW,empty)(space);
 		FN(PW,free)(pw);
 		return res;
 	}
