@@ -1,6 +1,7 @@
 /*
  * Copyright 2008-2009 Katholieke Universiteit Leuven
  * Copyright 2010      INRIA Saclay
+ * Copyright 2013-2014 Ecole Normale Superieure
  *
  * Use of this software is governed by the MIT license
  *
@@ -8,6 +9,7 @@
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
  * and INRIA Saclay - Ile-de-France, Parc Club Orsay Universite,
  * ZAC des vignes, 4 rue Jacques Monod, 91893 Orsay, France 
+ * and Ecole Normale Superieure, 45 rue d’Ulm, 75230 Paris, France
  */
 
 #include <stdlib.h>
@@ -63,23 +65,23 @@ static __isl_give isl_space *mark_as_set(__isl_take isl_space *space)
 
 /* Is the space that of a set?
  */
-int isl_space_is_set(__isl_keep isl_space *space)
+isl_bool isl_space_is_set(__isl_keep isl_space *space)
 {
 	if (!space)
-		return -1;
+		return isl_bool_error;
 	if (space->n_in != 0 || space->nested[0])
-		return 0;
+		return isl_bool_false;
 	if (space->tuple_id[0] != &isl_id_none)
-		return 0;
-	return 1;
+		return isl_bool_false;
+	return isl_bool_true;
 }
 
 /* Is the given space that of a map?
  */
-int isl_space_is_map(__isl_keep isl_space *space)
+isl_bool isl_space_is_map(__isl_keep isl_space *space)
 {
 	if (!space)
-		return -1;
+		return isl_bool_error;
 	return space->tuple_id[0] != &isl_id_none &&
 		space->tuple_id[1] != &isl_id_none;
 }
@@ -107,18 +109,18 @@ static __isl_give isl_space *mark_as_params(isl_space *space)
 
 /* Is the space that of a parameter domain?
  */
-int isl_space_is_params(__isl_keep isl_space *space)
+isl_bool isl_space_is_params(__isl_keep isl_space *space)
 {
 	if (!space)
-		return -1;
+		return isl_bool_error;
 	if (space->n_in != 0 || space->nested[0] ||
 	    space->n_out != 0 || space->nested[1])
-		return 0;
+		return isl_bool_false;
 	if (space->tuple_id[0] != &isl_id_none)
-		return 0;
+		return isl_bool_false;
 	if (space->tuple_id[1] != &isl_id_none)
-		return 0;
-	return 1;
+		return isl_bool_false;
+	return isl_bool_true;
 }
 
 /* Create a space for a parameter domain.
@@ -336,28 +338,28 @@ __isl_give isl_space *isl_space_copy(__isl_keep isl_space *dim)
 	return dim;
 }
 
-void *isl_space_free(__isl_take isl_space *dim)
+__isl_null isl_space *isl_space_free(__isl_take isl_space *space)
 {
 	int i;
 
-	if (!dim)
+	if (!space)
 		return NULL;
 
-	if (--dim->ref > 0)
+	if (--space->ref > 0)
 		return NULL;
 
-	isl_id_free(dim->tuple_id[0]);
-	isl_id_free(dim->tuple_id[1]);
+	isl_id_free(space->tuple_id[0]);
+	isl_id_free(space->tuple_id[1]);
 
-	isl_space_free(dim->nested[0]);
-	isl_space_free(dim->nested[1]);
+	isl_space_free(space->nested[0]);
+	isl_space_free(space->nested[1]);
 
-	for (i = 0; i < dim->n_id; ++i)
-		isl_id_free(dim->ids[i]);
-	free(dim->ids);
-	isl_ctx_deref(dim->ctx);
+	for (i = 0; i < space->n_id; ++i)
+		isl_id_free(space->ids[i]);
+	free(space->ids);
+	isl_ctx_deref(space->ctx);
 	
-	free(dim);
+	free(space);
 
 	return NULL;
 }
@@ -403,10 +405,11 @@ static int space_can_have_id(__isl_keep isl_space *space,
 
 /* Does the tuple have an id?
  */
-int isl_space_has_tuple_id(__isl_keep isl_space *dim, enum isl_dim_type type)
+isl_bool isl_space_has_tuple_id(__isl_keep isl_space *dim,
+	enum isl_dim_type type)
 {
 	if (!space_can_have_id(dim, type))
-		return -1;
+		return isl_bool_error;
 	return dim->tuple_id[type - isl_dim_in] != NULL;
 }
 
@@ -534,11 +537,11 @@ error:
 	return NULL;
 }
 
-int isl_space_has_dim_id(__isl_keep isl_space *dim,
+isl_bool isl_space_has_dim_id(__isl_keep isl_space *dim,
 	enum isl_dim_type type, unsigned pos)
 {
 	if (!dim)
-		return -1;
+		return isl_bool_error;
 	return get_id(dim, type, pos) != NULL;
 }
 
@@ -576,13 +579,13 @@ error:
 
 /* Does the tuple have a name?
  */
-int isl_space_has_tuple_name(__isl_keep isl_space *space,
+isl_bool isl_space_has_tuple_name(__isl_keep isl_space *space,
 	enum isl_dim_type type)
 {
 	isl_id *id;
 
 	if (!space_can_have_id(space, type))
-		return -1;
+		return isl_bool_error;
 	id = space->tuple_id[type - isl_dim_in];
 	return id && id->name;
 }
@@ -620,13 +623,13 @@ error:
 
 /* Does the given dimension have a name?
  */
-int isl_space_has_dim_name(__isl_keep isl_space *space,
+isl_bool isl_space_has_dim_name(__isl_keep isl_space *space,
 	enum isl_dim_type type, unsigned pos)
 {
 	isl_id *id;
 
 	if (!space)
-		return -1;
+		return isl_bool_error;
 	id = get_id(space, type, pos);
 	return id && id->name;
 }
@@ -677,6 +680,65 @@ int isl_space_find_dim_by_name(__isl_keep isl_space *space,
 	return -1;
 }
 
+/* Reset the user pointer on all identifiers of parameters and tuples
+ * of "space".
+ */
+__isl_give isl_space *isl_space_reset_user(__isl_take isl_space *space)
+{
+	int i;
+	isl_ctx *ctx;
+	isl_id *id;
+	const char *name;
+
+	if (!space)
+		return NULL;
+
+	ctx = isl_space_get_ctx(space);
+
+	for (i = 0; i < space->nparam && i < space->n_id; ++i) {
+		if (!isl_id_get_user(space->ids[i]))
+			continue;
+		space = isl_space_cow(space);
+		if (!space)
+			return NULL;
+		name = isl_id_get_name(space->ids[i]);
+		id = isl_id_alloc(ctx, name, NULL);
+		isl_id_free(space->ids[i]);
+		space->ids[i] = id;
+		if (!id)
+			return isl_space_free(space);
+	}
+
+	for (i = 0; i < 2; ++i) {
+		if (!space->tuple_id[i])
+			continue;
+		if (!isl_id_get_user(space->tuple_id[i]))
+			continue;
+		space = isl_space_cow(space);
+		if (!space)
+			return NULL;
+		name = isl_id_get_name(space->tuple_id[i]);
+		id = isl_id_alloc(ctx, name, NULL);
+		isl_id_free(space->tuple_id[i]);
+		space->tuple_id[i] = id;
+		if (!id)
+			return isl_space_free(space);
+	}
+
+	for (i = 0; i < 2; ++i) {
+		if (!space->nested[i])
+			continue;
+		space = isl_space_cow(space);
+		if (!space)
+			return NULL;
+		space->nested[i] = isl_space_reset_user(space->nested[i]);
+		if (!space->nested[i])
+			return isl_space_free(space);
+	}
+
+	return space;
+}
+
 static __isl_keep isl_id *tuple_id(__isl_keep isl_space *dim,
 	enum isl_dim_type type)
 {
@@ -701,33 +763,70 @@ static __isl_keep isl_space *nested(__isl_keep isl_space *dim,
 	return NULL;
 }
 
-int isl_space_tuple_match(__isl_keep isl_space *dim1, enum isl_dim_type dim1_type,
-			__isl_keep isl_space *dim2, enum isl_dim_type dim2_type)
+/* Are the two spaces the same, apart from positions and names of parameters?
+ */
+static int isl_space_has_equal_tuples(__isl_keep isl_space *space1,
+	__isl_keep isl_space *space2)
+{
+	if (!space1 || !space2)
+		return -1;
+	if (space1 == space2)
+		return 1;
+	return isl_space_tuple_is_equal(space1, isl_dim_in,
+					space2, isl_dim_in) &&
+	       isl_space_tuple_is_equal(space1, isl_dim_out,
+					space2, isl_dim_out);
+}
+
+/* Check if the tuple of type "type1" of "space1" is the same as
+ * the tuple of type "type2" of "space2".
+ *
+ * That is, check if the tuples have the same identifier, the same dimension
+ * and the same internal structure.
+ * The identifiers of the dimensions inside the tuples do not affect the result.
+ *
+ * Note that this function only checks the tuples themselves.
+ * If nested tuples are involved, then we need to be careful not
+ * to have result affected by possibly differing parameters
+ * in those nested tuples.
+ */
+isl_bool isl_space_tuple_is_equal(__isl_keep isl_space *space1,
+	enum isl_dim_type type1, __isl_keep isl_space *space2,
+	enum isl_dim_type type2)
 {
 	isl_id *id1, *id2;
 	isl_space *nested1, *nested2;
 
-	if (!dim1 || !dim2)
-		return -1;
+	if (!space1 || !space2)
+		return isl_bool_error;
 
-	if (dim1 == dim2 && dim1_type == dim2_type)
-		return 1;
+	if (space1 == space2 && type1 == type2)
+		return isl_bool_true;
 
-	if (n(dim1, dim1_type) != n(dim2, dim2_type))
-		return 0;
-	id1 = tuple_id(dim1, dim1_type);
-	id2 = tuple_id(dim2, dim2_type);
+	if (n(space1, type1) != n(space2, type2))
+		return isl_bool_false;
+	id1 = tuple_id(space1, type1);
+	id2 = tuple_id(space2, type2);
 	if (!id1 ^ !id2)
-		return 0;
+		return isl_bool_false;
 	if (id1 && id1 != id2)
-		return 0;
-	nested1 = nested(dim1, dim1_type);
-	nested2 = nested(dim2, dim2_type);
+		return isl_bool_false;
+	nested1 = nested(space1, type1);
+	nested2 = nested(space2, type2);
 	if (!nested1 ^ !nested2)
-		return 0;
-	if (nested1 && !isl_space_is_equal(nested1, nested2))
-		return 0;
-	return 1;
+		return isl_bool_false;
+	if (nested1 && !isl_space_has_equal_tuples(nested1, nested2))
+		return isl_bool_false;
+	return isl_bool_true;
+}
+
+/* This is the old, undocumented, name for isl_space_tuple_is_equal.
+ * It will be removed at some point.
+ */
+int isl_space_tuple_match(__isl_keep isl_space *space1, enum isl_dim_type type1,
+	__isl_keep isl_space *space2, enum isl_dim_type type2)
+{
+	return isl_space_tuple_is_equal(space1, type1, space2, type2);
 }
 
 static int match(__isl_keep isl_space *dim1, enum isl_dim_type dim1_type,
@@ -738,7 +837,7 @@ static int match(__isl_keep isl_space *dim1, enum isl_dim_type dim1_type,
 	if (dim1 == dim2 && dim1_type == dim2_type)
 		return 1;
 
-	if (!isl_space_tuple_match(dim1, dim1_type, dim2, dim2_type))
+	if (!isl_space_tuple_is_equal(dim1, dim1_type, dim2, dim2_type))
 		return 0;
 
 	if (!dim1->ids && !dim2->ids)
@@ -769,53 +868,60 @@ static void get_ids(__isl_keep isl_space *dim, enum isl_dim_type type,
 		ids[i] = get_id(dim, type, first + i);
 }
 
-__isl_give isl_space *isl_space_extend(__isl_take isl_space *dim,
+__isl_give isl_space *isl_space_extend(__isl_take isl_space *space,
 			unsigned nparam, unsigned n_in, unsigned n_out)
 {
 	isl_id **ids = NULL;
 
-	if (!dim)
+	if (!space)
 		return NULL;
-	if (dim->nparam == nparam && dim->n_in == n_in && dim->n_out == n_out)
-		return dim;
+	if (space->nparam == nparam &&
+	    space->n_in == n_in && space->n_out == n_out)
+		return space;
 
-	isl_assert(dim->ctx, dim->nparam <= nparam, goto error);
-	isl_assert(dim->ctx, dim->n_in <= n_in, goto error);
-	isl_assert(dim->ctx, dim->n_out <= n_out, goto error);
+	isl_assert(space->ctx, space->nparam <= nparam, goto error);
+	isl_assert(space->ctx, space->n_in <= n_in, goto error);
+	isl_assert(space->ctx, space->n_out <= n_out, goto error);
 
-	dim = isl_space_cow(dim);
-	if (!dim)
+	space = isl_space_cow(space);
+	if (!space)
 		goto error;
 
-	if (dim->ids) {
-		ids = isl_calloc_array(dim->ctx, isl_id *,
-					 nparam + n_in + n_out);
+	if (space->ids) {
+		unsigned n;
+		n = nparam + n_in + n_out;
+		if (n < nparam || n < n_in || n < n_out)
+			isl_die(isl_space_get_ctx(space), isl_error_invalid,
+				"overflow in total number of dimensions",
+				goto error);
+		ids = isl_calloc_array(space->ctx, isl_id *, n);
 		if (!ids)
 			goto error;
-		get_ids(dim, isl_dim_param, 0, dim->nparam, ids);
-		get_ids(dim, isl_dim_in, 0, dim->n_in, ids + nparam);
-		get_ids(dim, isl_dim_out, 0, dim->n_out, ids + nparam + n_in);
-		free(dim->ids);
-		dim->ids = ids;
-		dim->n_id = nparam + n_in + n_out;
+		get_ids(space, isl_dim_param, 0, space->nparam, ids);
+		get_ids(space, isl_dim_in, 0, space->n_in, ids + nparam);
+		get_ids(space, isl_dim_out, 0, space->n_out,
+			ids + nparam + n_in);
+		free(space->ids);
+		space->ids = ids;
+		space->n_id = nparam + n_in + n_out;
 	}
-	dim->nparam = nparam;
-	dim->n_in = n_in;
-	dim->n_out = n_out;
+	space->nparam = nparam;
+	space->n_in = n_in;
+	space->n_out = n_out;
 
-	return dim;
+	return space;
 error:
 	free(ids);
-	isl_space_free(dim);
+	isl_space_free(space);
 	return NULL;
 }
 
 __isl_give isl_space *isl_space_add_dims(__isl_take isl_space *dim,
 	enum isl_dim_type type, unsigned n)
 {
+	dim = isl_space_reset(dim, type);
 	if (!dim)
 		return NULL;
-	dim = isl_space_reset(dim, type);
 	switch (type) {
 	case isl_dim_param:
 		dim = isl_space_extend(dim,
@@ -941,8 +1047,10 @@ __isl_give isl_space *isl_space_move_dims(__isl_take isl_space *dim,
 
 	if (!dim)
 		return NULL;
-	if (n == 0)
-		return dim;
+	if (n == 0) {
+		dim = isl_space_reset(dim, src_type);
+		return isl_space_reset(dim, dst_type);
+	}
 
 	isl_assert(dim->ctx, src_pos + n <= isl_space_dim(dim, src_type),
 		goto error);
@@ -1040,7 +1148,7 @@ __isl_give isl_space *isl_space_join(__isl_take isl_space *left,
 	isl_assert(left->ctx, match(left, isl_dim_param, right, isl_dim_param),
 			goto error);
 	isl_assert(left->ctx,
-		isl_space_tuple_match(left, isl_dim_out, right, isl_dim_in),
+		isl_space_tuple_is_equal(left, isl_dim_out, right, isl_dim_in),
 		goto error);
 
 	dim = isl_space_alloc(left->ctx, left->nparam, left->n_in, right->n_out);
@@ -1074,14 +1182,27 @@ error:
 	return NULL;
 }
 
+/* Given two map spaces { A -> C } and { B -> D }, construct the space
+ * { [A -> B] -> [C -> D] }.
+ * Given two set spaces { A } and { B }, construct the space { [A -> B] }.
+ */
 __isl_give isl_space *isl_space_product(__isl_take isl_space *left,
 	__isl_take isl_space *right)
 {
 	isl_space *dom1, *dom2, *nest1, *nest2;
+	int is_set;
 
 	if (!left || !right)
 		goto error;
 
+	is_set = isl_space_is_set(left);
+	if (is_set != isl_space_is_set(right))
+		isl_die(isl_space_get_ctx(left), isl_error_invalid,
+			"expecting either two set spaces or two map spaces",
+			goto error);
+	if (is_set)
+		return isl_space_range_product(left, right);
+
 	isl_assert(left->ctx, match(left, isl_dim_param, right, isl_dim_param),
 			goto error);
 
@@ -1114,7 +1235,7 @@ __isl_give isl_space *isl_space_domain_product(__isl_take isl_space *left,
 	if (!match(left, isl_dim_param, right, isl_dim_param))
 		isl_die(left->ctx, isl_error_invalid,
 			"parameters need to match", goto error);
-	if (!isl_space_tuple_match(left, isl_dim_out, right, isl_dim_out))
+	if (!isl_space_tuple_is_equal(left, isl_dim_out, right, isl_dim_out))
 		isl_die(left->ctx, isl_error_invalid,
 			"ranges need to match", goto error);
 
@@ -1141,7 +1262,7 @@ __isl_give isl_space *isl_space_range_product(__isl_take isl_space *left,
 
 	isl_assert(left->ctx, match(left, isl_dim_param, right, isl_dim_param),
 			goto error);
-	if (!isl_space_tuple_match(left, isl_dim_in, right, isl_dim_in))
+	if (!isl_space_tuple_is_equal(left, isl_dim_in, right, isl_dim_in))
 		isl_die(left->ctx, isl_error_invalid,
 			"domains need to match", goto error);
 
@@ -1158,6 +1279,178 @@ error:
 	return NULL;
 }
 
+/* Given a space of the form [A -> B] -> [C -> D], return the space A -> C.
+ */
+__isl_give isl_space *isl_space_factor_domain(__isl_take isl_space *space)
+{
+	space = isl_space_domain_factor_domain(space);
+	space = isl_space_range_factor_domain(space);
+	return space;
+}
+
+/* Given a space of the form [A -> B] -> [C -> D], return the space B -> D.
+ */
+__isl_give isl_space *isl_space_factor_range(__isl_take isl_space *space)
+{
+	space = isl_space_domain_factor_range(space);
+	space = isl_space_range_factor_range(space);
+	return space;
+}
+
+/* Given a space of the form [A -> B] -> C, return the space A -> C.
+ */
+__isl_give isl_space *isl_space_domain_factor_domain(
+	__isl_take isl_space *space)
+{
+	isl_space *nested;
+	isl_space *domain;
+
+	if (!space)
+		return NULL;
+	if (!isl_space_domain_is_wrapping(space))
+		isl_die(isl_space_get_ctx(space), isl_error_invalid,
+			"domain not a product", return isl_space_free(space));
+
+	nested = space->nested[0];
+	domain = isl_space_copy(space);
+	domain = isl_space_drop_dims(domain, isl_dim_in,
+					nested->n_in, nested->n_out);
+	if (!domain)
+		return isl_space_free(space);
+	if (nested->tuple_id[0]) {
+		domain->tuple_id[0] = isl_id_copy(nested->tuple_id[0]);
+		if (!domain->tuple_id[0])
+			goto error;
+	}
+	if (nested->nested[0]) {
+		domain->nested[0] = isl_space_copy(nested->nested[0]);
+		if (!domain->nested[0])
+			goto error;
+	}
+
+	isl_space_free(space);
+	return domain;
+error:
+	isl_space_free(space);
+	isl_space_free(domain);
+	return NULL;
+}
+
+/* Given a space of the form [A -> B] -> C, return the space B -> C.
+ */
+__isl_give isl_space *isl_space_domain_factor_range(
+	__isl_take isl_space *space)
+{
+	isl_space *nested;
+	isl_space *range;
+
+	if (!space)
+		return NULL;
+	if (!isl_space_domain_is_wrapping(space))
+		isl_die(isl_space_get_ctx(space), isl_error_invalid,
+			"domain not a product", return isl_space_free(space));
+
+	nested = space->nested[0];
+	range = isl_space_copy(space);
+	range = isl_space_drop_dims(range, isl_dim_in, 0, nested->n_in);
+	if (!range)
+		return isl_space_free(space);
+	if (nested->tuple_id[1]) {
+		range->tuple_id[0] = isl_id_copy(nested->tuple_id[1]);
+		if (!range->tuple_id[0])
+			goto error;
+	}
+	if (nested->nested[1]) {
+		range->nested[0] = isl_space_copy(nested->nested[1]);
+		if (!range->nested[0])
+			goto error;
+	}
+
+	isl_space_free(space);
+	return range;
+error:
+	isl_space_free(space);
+	isl_space_free(range);
+	return NULL;
+}
+
+/* Given a space of the form A -> [B -> C], return the space A -> B.
+ */
+__isl_give isl_space *isl_space_range_factor_domain(
+	__isl_take isl_space *space)
+{
+	isl_space *nested;
+	isl_space *domain;
+
+	if (!space)
+		return NULL;
+	if (!isl_space_range_is_wrapping(space))
+		isl_die(isl_space_get_ctx(space), isl_error_invalid,
+			"range not a product", return isl_space_free(space));
+
+	nested = space->nested[1];
+	domain = isl_space_copy(space);
+	domain = isl_space_drop_dims(domain, isl_dim_out,
+					nested->n_in, nested->n_out);
+	if (!domain)
+		return isl_space_free(space);
+	if (nested->tuple_id[0]) {
+		domain->tuple_id[1] = isl_id_copy(nested->tuple_id[0]);
+		if (!domain->tuple_id[1])
+			goto error;
+	}
+	if (nested->nested[0]) {
+		domain->nested[1] = isl_space_copy(nested->nested[0]);
+		if (!domain->nested[1])
+			goto error;
+	}
+
+	isl_space_free(space);
+	return domain;
+error:
+	isl_space_free(space);
+	isl_space_free(domain);
+	return NULL;
+}
+
+/* Given a space of the form A -> [B -> C], return the space A -> C.
+ */
+__isl_give isl_space *isl_space_range_factor_range(
+	__isl_take isl_space *space)
+{
+	isl_space *nested;
+	isl_space *range;
+
+	if (!space)
+		return NULL;
+	if (!isl_space_range_is_wrapping(space))
+		isl_die(isl_space_get_ctx(space), isl_error_invalid,
+			"range not a product", return isl_space_free(space));
+
+	nested = space->nested[1];
+	range = isl_space_copy(space);
+	range = isl_space_drop_dims(range, isl_dim_out, 0, nested->n_in);
+	if (!range)
+		return isl_space_free(space);
+	if (nested->tuple_id[1]) {
+		range->tuple_id[1] = isl_id_copy(nested->tuple_id[1]);
+		if (!range->tuple_id[1])
+			goto error;
+	}
+	if (nested->nested[1]) {
+		range->nested[1] = isl_space_copy(nested->nested[1]);
+		if (!range->nested[1])
+			goto error;
+	}
+
+	isl_space_free(space);
+	return range;
+error:
+	isl_space_free(space);
+	isl_space_free(range);
+	return NULL;
+}
+
 __isl_give isl_space *isl_space_map_from_set(__isl_take isl_space *dim)
 {
 	isl_ctx *ctx;
@@ -1291,7 +1584,9 @@ __isl_give isl_space *isl_space_drop_dims(__isl_take isl_space *dim,
 		isl_die(dim->ctx, isl_error_invalid,
 			"cannot drop dimensions of specified type", goto error);
 
-	isl_assert(dim->ctx, first + num <= n(dim, type), goto error);
+	if (first + num > n(dim, type) || first + num < first)
+		isl_die(isl_space_get_ctx(dim), isl_error_invalid,
+			"index out of bounds", return isl_space_free(dim));
 	dim = isl_space_cow(dim);
 	if (!dim)
 		goto error;
@@ -1401,6 +1696,32 @@ error:
 	return NULL;
 }
 
+/* Given a map space A -> B, return the map space [A -> B] -> A.
+ */
+__isl_give isl_space *isl_space_domain_map(__isl_take isl_space *space)
+{
+	isl_space *domain;
+
+	domain = isl_space_from_range(isl_space_domain(isl_space_copy(space)));
+	space = isl_space_from_domain(isl_space_wrap(space));
+	space = isl_space_join(space, domain);
+
+	return space;
+}
+
+/* Given a map space A -> B, return the map space [A -> B] -> B.
+ */
+__isl_give isl_space *isl_space_range_map(__isl_take isl_space *space)
+{
+	isl_space *range;
+
+	range = isl_space_from_range(isl_space_range(isl_space_copy(space)));
+	space = isl_space_from_domain(isl_space_wrap(space));
+	space = isl_space_join(space, range);
+
+	return space;
+}
+
 __isl_give isl_space *isl_space_params(__isl_take isl_space *space)
 {
 	if (isl_space_is_params(space))
@@ -1468,15 +1789,16 @@ __isl_give isl_space *isl_space_underlying(__isl_take isl_space *dim,
 
 /* Are the two spaces the same, including positions and names of parameters?
  */
-int isl_space_is_equal(__isl_keep isl_space *dim1, __isl_keep isl_space *dim2)
+isl_bool isl_space_is_equal(__isl_keep isl_space *dim1,
+	__isl_keep isl_space *dim2)
 {
 	if (!dim1 || !dim2)
-		return -1;
+		return isl_bool_error;
 	if (dim1 == dim2)
-		return 1;
+		return isl_bool_true;
 	return match(dim1, isl_dim_param, dim2, isl_dim_param) &&
-	       isl_space_tuple_match(dim1, isl_dim_in, dim2, isl_dim_in) &&
-	       isl_space_tuple_match(dim1, isl_dim_out, dim2, isl_dim_out);
+	       isl_space_tuple_is_equal(dim1, isl_dim_in, dim2, isl_dim_in) &&
+	       isl_space_tuple_is_equal(dim1, isl_dim_out, dim2, isl_dim_out);
 }
 
 /* Is space1 equal to the domain of space2?
@@ -1484,26 +1806,27 @@ int isl_space_is_equal(__isl_keep isl_space *dim1, __isl_keep isl_space *dim2)
  * In the internal version we also allow space2 to be the space of a set,
  * provided space1 is a parameter space.
  */
-int isl_space_is_domain_internal(__isl_keep isl_space *space1,
+isl_bool isl_space_is_domain_internal(__isl_keep isl_space *space1,
 	__isl_keep isl_space *space2)
 {
 	if (!space1 || !space2)
-		return -1;
+		return isl_bool_error;
 	if (!isl_space_is_set(space1))
-		return 0;
+		return isl_bool_false;
 	return match(space1, isl_dim_param, space2, isl_dim_param) &&
-	       isl_space_tuple_match(space1, isl_dim_set, space2, isl_dim_in);
+	       isl_space_tuple_is_equal(space1, isl_dim_set,
+					space2, isl_dim_in);
 }
 
 /* Is space1 equal to the domain of space2?
  */
-int isl_space_is_domain(__isl_keep isl_space *space1,
+isl_bool isl_space_is_domain(__isl_keep isl_space *space1,
 	__isl_keep isl_space *space2)
 {
 	if (!space2)
-		return -1;
+		return isl_bool_error;
 	if (!isl_space_is_map(space2))
-		return 0;
+		return isl_bool_false;
 	return isl_space_is_domain_internal(space1, space2);
 }
 
@@ -1512,26 +1835,27 @@ int isl_space_is_domain(__isl_keep isl_space *space1,
  * In the internal version, space2 is allowed to be the space of a set,
  * in which case it should be equal to space1.
  */
-int isl_space_is_range_internal(__isl_keep isl_space *space1,
+isl_bool isl_space_is_range_internal(__isl_keep isl_space *space1,
 	__isl_keep isl_space *space2)
 {
 	if (!space1 || !space2)
-		return -1;
+		return isl_bool_error;
 	if (!isl_space_is_set(space1))
-		return 0;
+		return isl_bool_false;
 	return match(space1, isl_dim_param, space2, isl_dim_param) &&
-	       isl_space_tuple_match(space1, isl_dim_set, space2, isl_dim_out);
+	       isl_space_tuple_is_equal(space1, isl_dim_set,
+					space2, isl_dim_out);
 }
 
 /* Is space1 equal to the range of space2?
  */
-int isl_space_is_range(__isl_keep isl_space *space1,
+isl_bool isl_space_is_range(__isl_keep isl_space *space1,
 	__isl_keep isl_space *space2)
 {
 	if (!space2)
-		return -1;
+		return isl_bool_error;
 	if (!isl_space_is_map(space2))
-		return 0;
+		return isl_bool_false;
 	return isl_space_is_range_internal(space1, space2);
 }
 
@@ -1583,17 +1907,43 @@ uint32_t isl_space_get_hash(__isl_keep isl_space *dim)
 	return hash;
 }
 
-int isl_space_is_wrapping(__isl_keep isl_space *dim)
+isl_bool isl_space_is_wrapping(__isl_keep isl_space *dim)
 {
 	if (!dim)
-		return -1;
+		return isl_bool_error;
 
 	if (!isl_space_is_set(dim))
-		return 0;
+		return isl_bool_false;
 
 	return dim->nested[1] != NULL;
 }
 
+/* Is "space" the space of a map where the domain is a wrapped map space?
+ */
+isl_bool isl_space_domain_is_wrapping(__isl_keep isl_space *space)
+{
+	if (!space)
+		return isl_bool_error;
+
+	if (isl_space_is_set(space))
+		return isl_bool_false;
+
+	return space->nested[0] != NULL;
+}
+
+/* Is "space" the space of a map where the range is a wrapped map space?
+ */
+isl_bool isl_space_range_is_wrapping(__isl_keep isl_space *space)
+{
+	if (!space)
+		return isl_bool_error;
+
+	if (isl_space_is_set(space))
+		return isl_bool_false;
+
+	return space->nested[1] != NULL;
+}
+
 __isl_give isl_space *isl_space_wrap(__isl_take isl_space *dim)
 {
 	isl_space *wrap;
@@ -1774,10 +2124,10 @@ __isl_give isl_space *isl_space_lift(__isl_take isl_space *dim, unsigned n_local
 	return dim;
 }
 
-int isl_space_can_zip(__isl_keep isl_space *dim)
+isl_bool isl_space_can_zip(__isl_keep isl_space *dim)
 {
 	if (!dim)
-		return -1;
+		return isl_bool_error;
 
 	return dim->nested[0] && dim->nested[1];
 }
@@ -1813,10 +2163,10 @@ error:
 /* Can we apply isl_space_curry to "space"?
  * That is, does it have a nested relation in its domain?
  */
-int isl_space_can_curry(__isl_keep isl_space *space)
+isl_bool isl_space_can_curry(__isl_keep isl_space *space)
 {
 	if (!space)
-		return -1;
+		return isl_bool_error;
 
 	return !!space->nested[0];
 }
@@ -1852,10 +2202,10 @@ error:
 /* Can we apply isl_space_uncurry to "space"?
  * That is, does it have a nested relation in its range?
  */
-int isl_space_can_uncurry(__isl_keep isl_space *space)
+isl_bool isl_space_can_uncurry(__isl_keep isl_space *space)
 {
 	if (!space)
-		return -1;
+		return isl_bool_error;
 
 	return !!space->nested[1];
 }
@@ -1968,3 +2318,71 @@ error:
 	isl_space_free(space);
 	return NULL;
 }
+
+/* Compare the "type" dimensions of two isl_spaces.
+ *
+ * The order is fairly arbitrary.
+ */
+static int isl_space_cmp_type(__isl_keep isl_space *space1,
+	__isl_keep isl_space *space2, enum isl_dim_type type)
+{
+	int cmp;
+	isl_space *nested1, *nested2;
+
+	if (isl_space_dim(space1, type) != isl_space_dim(space2, type))
+		return isl_space_dim(space1, type) -
+			isl_space_dim(space2, type);
+
+	cmp = isl_id_cmp(tuple_id(space1, type), tuple_id(space2, type));
+	if (cmp != 0)
+		return cmp;
+
+	nested1 = nested(space1, type);
+	nested2 = nested(space2, type);
+	if (!nested1 != !nested2)
+		return !nested1 - !nested2;
+
+	if (nested1)
+		return isl_space_cmp(nested1, nested2);
+
+	return 0;
+}
+
+/* Compare two isl_spaces.
+ *
+ * The order is fairly arbitrary.
+ */
+int isl_space_cmp(__isl_keep isl_space *space1, __isl_keep isl_space *space2)
+{
+	int i;
+	int cmp;
+
+	if (space1 == space2)
+		return 0;
+	if (!space1)
+		return -1;
+	if (!space2)
+		return 1;
+
+	cmp = isl_space_cmp_type(space1, space2, isl_dim_param);
+	if (cmp != 0)
+		return cmp;
+	cmp = isl_space_cmp_type(space1, space2, isl_dim_in);
+	if (cmp != 0)
+		return cmp;
+	cmp = isl_space_cmp_type(space1, space2, isl_dim_out);
+	if (cmp != 0)
+		return cmp;
+
+	if (!space1->ids && !space2->ids)
+		return 0;
+
+	for (i = 0; i < n(space1, isl_dim_param); ++i) {
+		cmp = isl_id_cmp(get_id(space1, isl_dim_param, i),
+				 get_id(space2, isl_dim_param, i));
+		if (cmp != 0)
+			return cmp;
+	}
+
+	return 0;
+}
