@@ -29,9 +29,9 @@ __isl_give isl_space *isl_space_underlying(__isl_take isl_space *dim,
 
 uint32_t isl_space_get_hash(__isl_keep isl_space *dim);
 
-int isl_space_is_domain_internal(__isl_keep isl_space *space1,
+isl_bool isl_space_is_domain_internal(__isl_keep isl_space *space1,
 	__isl_keep isl_space *space2);
-int isl_space_is_range_internal(__isl_keep isl_space *space1,
+isl_bool isl_space_is_range_internal(__isl_keep isl_space *space1,
 	__isl_keep isl_space *space2);
 
 __isl_give isl_space *isl_space_as_set_space(__isl_take isl_space *dim);
@@ -55,4 +55,6 @@ __isl_give isl_space *isl_space_lift(__isl_take isl_space *dim, unsigned n_local
 __isl_give isl_space *isl_space_extend_domain_with_range(
 	__isl_take isl_space *domain, __isl_take isl_space *model);
 
+int isl_space_cmp(__isl_keep isl_space *space1, __isl_keep isl_space *space2);
+
 #endif
