@@ -1,17 +1,23 @@
 /*
  * Copyright 2008-2009 Katholieke Universiteit Leuven
+ * Copyright 2013      Ecole Normale Superieure
+ * Copyright 2014      INRIA Rocquencourt
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, K.U.Leuven, Departement
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
+ * and Ecole Normale Superieure, 45 rue d'Ulm, 75230 Paris, France
+ * and Inria Paris - Rocquencourt, Domaine de Voluceau - Rocquencourt,
+ * B.P. 105 - 78153 Le Chesnay, France
  */
 
 #include <isl_ctx_private.h>
 #include <isl_mat_private.h>
+#include <isl_vec_private.h>
 #include "isl_map_private.h"
 #include "isl_tab.h"
-#include <isl/seq.h>
+#include <isl_seq.h>
 #include <isl_config.h>
 
 /*
@@ -34,16 +40,16 @@ struct isl_tab *isl_tab_alloc(struct isl_ctx *ctx,
 	if (!tab->mat)
 		goto error;
 	tab->var = isl_alloc_array(ctx, struct isl_tab_var, n_var);
-	if (!tab->var)
+	if (n_var && !tab->var)
 		goto error;
 	tab->con = isl_alloc_array(ctx, struct isl_tab_var, n_row);
-	if (!tab->con)
+	if (n_row && !tab->con)
 		goto error;
 	tab->col_var = isl_alloc_array(ctx, int, n_var);
-	if (!tab->col_var)
+	if (n_var && !tab->col_var)
 		goto error;
 	tab->row_var = isl_alloc_array(ctx, int, n_row);
-	if (!tab->row_var)
+	if (n_row && !tab->row_var)
 		goto error;
 	for (i = 0; i < n_var; ++i) {
 		tab->var[i].index = i;
@@ -87,6 +93,11 @@ error:
 	return NULL;
 }
 
+isl_ctx *isl_tab_get_ctx(struct isl_tab *tab)
+{
+	return tab ? isl_mat_get_ctx(tab->mat) : NULL;
+}
+
 int isl_tab_extend_cons(struct isl_tab *tab, unsigned n_new)
 {
 	unsigned off;
@@ -144,7 +155,7 @@ int isl_tab_extend_vars(struct isl_tab *tab, unsigned n_new)
 		if (!var)
 			return -1;
 		tab->var = var;
-		tab->max_var += n_new;
+		tab->max_var = tab->n_var + n_new;
 	}
 
 	if (tab->mat->n_col < off + tab->n_col + n_new) {
@@ -164,15 +175,6 @@ int isl_tab_extend_vars(struct isl_tab *tab, unsigned n_new)
 	return 0;
 }
 
-struct isl_tab *isl_tab_extend(struct isl_tab *tab, unsigned n_new)
-{
-	if (isl_tab_extend_cons(tab, n_new) >= 0)
-		return tab;
-
-	isl_tab_free(tab);
-	return NULL;
-}
-
 static void free_undo_record(struct isl_tab_undo *undo)
 {
 	switch (undo->type) {
@@ -231,29 +233,29 @@ struct isl_tab *isl_tab_dup(struct isl_tab *tab)
 	if (!dup->mat)
 		goto error;
 	dup->var = isl_alloc_array(tab->mat->ctx, struct isl_tab_var, tab->max_var);
-	if (!dup->var)
+	if (tab->max_var && !dup->var)
 		goto error;
 	for (i = 0; i < tab->n_var; ++i)
 		dup->var[i] = tab->var[i];
 	dup->con = isl_alloc_array(tab->mat->ctx, struct isl_tab_var, tab->max_con);
-	if (!dup->con)
+	if (tab->max_con && !dup->con)
 		goto error;
 	for (i = 0; i < tab->n_con; ++i)
 		dup->con[i] = tab->con[i];
 	dup->col_var = isl_alloc_array(tab->mat->ctx, int, tab->mat->n_col - off);
-	if (!dup->col_var)
+	if ((tab->mat->n_col - off) && !dup->col_var)
 		goto error;
 	for (i = 0; i < tab->n_col; ++i)
 		dup->col_var[i] = tab->col_var[i];
 	dup->row_var = isl_alloc_array(tab->mat->ctx, int, tab->mat->n_row);
-	if (!dup->row_var)
+	if (tab->mat->n_row && !dup->row_var)
 		goto error;
 	for (i = 0; i < tab->n_row; ++i)
 		dup->row_var[i] = tab->row_var[i];
 	if (tab->row_sign) {
 		dup->row_sign = isl_alloc_array(tab->mat->ctx, enum isl_tab_row_sign,
 						tab->mat->n_row);
-		if (!dup->row_sign)
+		if (tab->mat->n_row && !dup->row_sign)
 			goto error;
 		for (i = 0; i < tab->n_row; ++i)
 			dup->row_sign[i] = tab->row_sign[i];
@@ -264,7 +266,7 @@ struct isl_tab *isl_tab_dup(struct isl_tab *tab)
 			goto error;
 		dup->sample_index = isl_alloc_array(tab->mat->ctx, int,
 							tab->samples->n_row);
-		if (!dup->sample_index)
+		if (tab->samples->n_row && !dup->sample_index)
 			goto error;
 		dup->n_sample = tab->n_sample;
 		dup->n_outside = tab->n_outside;
@@ -460,7 +462,7 @@ struct isl_tab *isl_tab_product(struct isl_tab *tab1, struct isl_tab *tab2)
 		goto error;
 	prod->var = isl_alloc_array(tab1->mat->ctx, struct isl_tab_var,
 					tab1->max_var + tab2->max_var);
-	if (!prod->var)
+	if ((tab1->max_var + tab2->max_var) && !prod->var)
 		goto error;
 	for (i = 0; i < tab1->n_var; ++i) {
 		prod->var[i] = tab1->var[i];
@@ -474,7 +476,7 @@ struct isl_tab *isl_tab_product(struct isl_tab *tab1, struct isl_tab *tab2)
 	}
 	prod->con = isl_alloc_array(tab1->mat->ctx, struct isl_tab_var,
 					tab1->max_con +  tab2->max_con);
-	if (!prod->con)
+	if ((tab1->max_con + tab2->max_con) && !prod->con)
 		goto error;
 	for (i = 0; i < tab1->n_con; ++i) {
 		prod->con[i] = tab1->con[i];
@@ -488,7 +490,7 @@ struct isl_tab *isl_tab_product(struct isl_tab *tab1, struct isl_tab *tab2)
 	}
 	prod->col_var = isl_alloc_array(tab1->mat->ctx, int,
 					tab1->n_col + tab2->n_col);
-	if (!prod->col_var)
+	if ((tab1->n_col + tab2->n_col) && !prod->col_var)
 		goto error;
 	for (i = 0; i < tab1->n_col; ++i) {
 		int pos = i < d1 ? i : i + d2;
@@ -505,7 +507,7 @@ struct isl_tab *isl_tab_product(struct isl_tab *tab1, struct isl_tab *tab2)
 	}
 	prod->row_var = isl_alloc_array(tab1->mat->ctx, int,
 					tab1->mat->n_row + tab2->mat->n_row);
-	if (!prod->row_var)
+	if ((tab1->mat->n_row + tab2->mat->n_row) && !prod->row_var)
 		goto error;
 	for (i = 0; i < tab1->n_row; ++i) {
 		int pos = i < r1 ? i : i + r2;
@@ -787,6 +789,8 @@ static int push_union(struct isl_tab *tab,
 {
 	struct isl_tab_undo *undo;
 
+	if (!tab)
+		return -1;
 	if (!tab->need_undo)
 		return 0;
 
@@ -827,7 +831,7 @@ int isl_tab_push_basis(struct isl_tab *tab)
 	union isl_tab_undo_val u;
 
 	u.col_var = isl_alloc_array(tab->mat->ctx, int, tab->n_col);
-	if (!u.col_var)
+	if (tab->n_col && !u.col_var)
 		return -1;
 	for (i = 0; i < tab->n_col; ++i)
 		u.col_var[i] = tab->col_var[i];
@@ -860,8 +864,7 @@ error:
 	return NULL;
 }
 
-struct isl_tab *isl_tab_add_sample(struct isl_tab *tab,
-	__isl_take isl_vec *sample)
+int isl_tab_add_sample(struct isl_tab *tab, __isl_take isl_vec *sample)
 {
 	if (!tab || !sample)
 		goto error;
@@ -884,11 +887,10 @@ struct isl_tab *isl_tab_add_sample(struct isl_tab *tab,
 	tab->sample_index[tab->n_sample] = tab->n_sample;
 	tab->n_sample++;
 
-	return tab;
+	return 0;
 error:
 	isl_vec_free(sample);
-	isl_tab_free(tab);
-	return NULL;
+	return -1;
 }
 
 struct isl_tab *isl_tab_drop_sample(struct isl_tab *tab, int s)
@@ -939,7 +941,7 @@ int isl_tab_mark_redundant(struct isl_tab *tab, int row)
 	struct isl_tab_var *var = isl_tab_var_from_row(tab, row);
 	var->is_redundant = 1;
 	isl_assert(tab->mat->ctx, row >= tab->n_redundant, return -1);
-	if (tab->need_undo || tab->row_var[row] >= 0) {
+	if (tab->preserve || tab->need_undo || tab->row_var[row] >= 0) {
 		if (tab->row_var[row] >= 0 && !var->is_nonneg) {
 			var->is_nonneg = 1;
 			if (isl_tab_push_var(tab, isl_tab_undo_nonneg, var) < 0)
@@ -958,6 +960,22 @@ int isl_tab_mark_redundant(struct isl_tab *tab, int row)
 	}
 }
 
+/* Mark "tab" as a rational tableau.
+ * If it wasn't marked as a rational tableau already and if we may
+ * need to undo changes, then arrange for the marking to be undone
+ * during the undo.
+ */
+int isl_tab_mark_rational(struct isl_tab *tab)
+{
+	if (!tab)
+		return -1;
+	if (!tab->rational && tab->need_undo)
+		if (isl_tab_push(tab, isl_tab_undo_rational) < 0)
+			return -1;
+	tab->rational = 1;
+	return 0;
+}
+
 int isl_tab_mark_empty(struct isl_tab *tab)
 {
 	if (!tab)
@@ -1092,14 +1110,14 @@ int isl_tab_pivot(struct isl_tab *tab, int row, int col)
 	int i, j;
 	int sgn;
 	int t;
+	isl_ctx *ctx;
 	struct isl_mat *mat = tab->mat;
 	struct isl_tab_var *var;
 	unsigned off = 2 + tab->M;
 
-	if (tab->mat->ctx->abort) {
-		isl_ctx_set_error(tab->mat->ctx, isl_error_abort);
+	ctx = isl_tab_get_ctx(tab);
+	if (isl_ctx_next_operation(ctx) < 0)
 		return -1;
-	}
 
 	isl_int_swap(mat->row[row][0], mat->row[row][off + col]);
 	sgn = isl_int_sgn(mat->row[row][0]);
@@ -1410,7 +1428,8 @@ static int row_at_most_neg_one(struct isl_tab *tab, int row)
 /* Return 1 if "var" can attain values <= -1.
  * Return 0 otherwise.
  *
- * The sample value of "var" is assumed to be non-negative when the
+ * If the variable "var" is supposed to be non-negative (is_nonneg is set),
+ * then the sample value of "var" is assumed to be non-negative when the
  * the function is called.  If 1 is returned then the constraint
  * is not redundant and the sample value is made non-negative again before
  * the function returns.
@@ -1448,7 +1467,7 @@ int isl_tab_min_at_most_neg_one(struct isl_tab *tab, struct isl_tab_var *var)
 	do {
 		find_pivot(tab, var, var, -1, &row, &col);
 		if (row == var->index) {
-			if (restore_row(tab, var) < -1)
+			if (var->is_nonneg && restore_row(tab, var) < -1)
 				return -1;
 			return 1;
 		}
@@ -1644,19 +1663,75 @@ int isl_tab_allocate_con(struct isl_tab *tab)
 	return r;
 }
 
-/* Add a variable to the tableau and allocate a column for it.
- * Return the index into the variable array "var".
+/* Move the entries in tab->var up one position, starting at "first",
+ * creating room for an extra entry at position "first".
+ * Since some of the entries of tab->row_var and tab->col_var contain
+ * indices into this array, they have to be updated accordingly.
  */
-int isl_tab_allocate_var(struct isl_tab *tab)
+static int var_insert_entry(struct isl_tab *tab, int first)
+{
+	int i;
+
+	if (tab->n_var >= tab->max_var)
+		isl_die(isl_tab_get_ctx(tab), isl_error_internal,
+			"not enough room for new variable", return -1);
+	if (first > tab->n_var)
+		isl_die(isl_tab_get_ctx(tab), isl_error_internal,
+			"invalid initial position", return -1);
+
+	for (i = tab->n_var - 1; i >= first; --i) {
+		tab->var[i + 1] = tab->var[i];
+		if (tab->var[i + 1].is_row)
+			tab->row_var[tab->var[i + 1].index]++;
+		else
+			tab->col_var[tab->var[i + 1].index]++;
+	}
+
+	tab->n_var++;
+
+	return 0;
+}
+
+/* Drop the entry at position "first" in tab->var, moving all
+ * subsequent entries down.
+ * Since some of the entries of tab->row_var and tab->col_var contain
+ * indices into this array, they have to be updated accordingly.
+ */
+static int var_drop_entry(struct isl_tab *tab, int first)
+{
+	int i;
+
+	if (first >= tab->n_var)
+		isl_die(isl_tab_get_ctx(tab), isl_error_internal,
+			"invalid initial position", return -1);
+
+	tab->n_var--;
+
+	for (i = first; i < tab->n_var; ++i) {
+		tab->var[i] = tab->var[i + 1];
+		if (tab->var[i + 1].is_row)
+			tab->row_var[tab->var[i].index]--;
+		else
+			tab->col_var[tab->var[i].index]--;
+	}
+
+	return 0;
+}
+
+/* Add a variable to the tableau at position "r" and allocate a column for it.
+ * Return the index into the variable array "var", i.e., "r",
+ * or -1 on error.
+ */
+int isl_tab_insert_var(struct isl_tab *tab, int r)
 {
-	int r;
 	int i;
 	unsigned off = 2 + tab->M;
 
 	isl_assert(tab->mat->ctx, tab->n_col < tab->mat->n_col, return -1);
-	isl_assert(tab->mat->ctx, tab->n_var < tab->max_var, return -1);
 
-	r = tab->n_var;
+	if (var_insert_entry(tab, r) < 0)
+		return -1;
+
 	tab->var[r].index = tab->n_col;
 	tab->var[r].is_row = 0;
 	tab->var[r].is_nonneg = 0;
@@ -1669,7 +1744,6 @@ int isl_tab_allocate_var(struct isl_tab *tab)
 	for (i = 0; i < tab->n_row; ++i)
 		isl_int_set_si(tab->mat->row[i][off + tab->n_col], 0);
 
-	tab->n_var++;
 	tab->n_col++;
 	if (isl_tab_push_var(tab, isl_tab_undo_allocate, &tab->var[r]) < 0)
 		return -1;
@@ -1677,6 +1751,17 @@ int isl_tab_allocate_var(struct isl_tab *tab)
 	return r;
 }
 
+/* Add a variable to the tableau and allocate a column for it.
+ * Return the index into the variable array "var".
+ */
+int isl_tab_allocate_var(struct isl_tab *tab)
+{
+	if (!tab)
+		return -1;
+
+	return isl_tab_insert_var(tab, tab->n_var);
+}
+
 /* Add a row to the tableau.  The row is given as an affine combination
  * of the original variables and needs to be expressed in terms of the
  * column variables.
@@ -1754,13 +1839,22 @@ static int drop_row(struct isl_tab *tab, int row)
 	return 0;
 }
 
+/* Drop the variable in column "col" along with the column.
+ * The column is removed first because it may need to be moved
+ * into the last position and this process requires
+ * the contents of the col_var array in a state
+ * before the removal of the variable.
+ */
 static int drop_col(struct isl_tab *tab, int col)
 {
-	isl_assert(tab->mat->ctx, tab->col_var[col] == tab->n_var - 1, return -1);
+	int var;
+
+	var = tab->col_var[col];
 	if (col != tab->n_col - 1)
 		swap_cols(tab, col, tab->n_col - 1);
 	tab->n_col--;
-	tab->n_var--;
+	if (var_drop_entry(tab, var) < 0)
+		return -1;
 	return 0;
 }
 
@@ -1983,12 +2077,9 @@ int isl_tab_add_eq(struct isl_tab *tab, isl_int *eq)
 	var = &tab->con[r];
 	row = var->index;
 	if (row_is_manifestly_zero(tab, row)) {
-		if (snap) {
-			if (isl_tab_rollback(tab, snap) < 0)
-				return -1;
-		} else
-			drop_row(tab, row);
-		return 0;
+		if (snap)
+			return isl_tab_rollback(tab, snap);
+		return drop_row(tab, row);
 	}
 
 	if (tab->bmap) {
@@ -2199,7 +2290,13 @@ int isl_tab_add_div(struct isl_tab *tab, __isl_keep isl_vec *div,
 	return r;
 }
 
-struct isl_tab *isl_tab_from_basic_map(struct isl_basic_map *bmap)
+/* If "track" is set, then we want to keep track of all constraints in tab
+ * in its bmap field.  This field is initialized from a copy of "bmap",
+ * so we need to make sure that all constraints in "bmap" also appear
+ * in the constructed tab.
+ */
+__isl_give struct isl_tab *isl_tab_from_basic_map(
+	__isl_keep isl_basic_map *bmap, int track)
 {
 	int i;
 	struct isl_tab *tab;
@@ -2211,11 +2308,12 @@ struct isl_tab *isl_tab_from_basic_map(struct isl_basic_map *bmap)
 			    isl_basic_map_total_dim(bmap), 0);
 	if (!tab)
 		return NULL;
+	tab->preserve = track;
 	tab->rational = ISL_F_ISSET(bmap, ISL_BASIC_MAP_RATIONAL);
 	if (ISL_F_ISSET(bmap, ISL_BASIC_MAP_EMPTY)) {
 		if (isl_tab_mark_empty(tab) < 0)
 			goto error;
-		return tab;
+		goto done;
 	}
 	for (i = 0; i < bmap->n_eq; ++i) {
 		tab = add_eq(tab, bmap->eq[i]);
@@ -2226,17 +2324,21 @@ struct isl_tab *isl_tab_from_basic_map(struct isl_basic_map *bmap)
 		if (isl_tab_add_ineq(tab, bmap->ineq[i]) < 0)
 			goto error;
 		if (tab->empty)
-			return tab;
+			goto done;
 	}
+done:
+	if (track && isl_tab_track_bmap(tab, isl_basic_map_copy(bmap)) < 0)
+		goto error;
 	return tab;
 error:
 	isl_tab_free(tab);
 	return NULL;
 }
 
-struct isl_tab *isl_tab_from_basic_set(struct isl_basic_set *bset)
+__isl_give struct isl_tab *isl_tab_from_basic_set(
+	__isl_keep isl_basic_set *bset, int track)
 {
-	return isl_tab_from_basic_map((struct isl_basic_map *)bset);
+	return isl_tab_from_basic_map(bset, track);
 }
 
 /* Construct a tableau corresponding to the recession cone of "bset".
@@ -2535,30 +2637,37 @@ static int cut_to_hyperplane(struct isl_tab *tab, struct isl_tab_var *var)
  * even after the relaxation, so we need to restore it.
  * We therefore prefer to pivot a column up to a row, if possible.
  */
-struct isl_tab *isl_tab_relax(struct isl_tab *tab, int con)
+int isl_tab_relax(struct isl_tab *tab, int con)
 {
 	struct isl_tab_var *var;
-	unsigned off = 2 + tab->M;
 
 	if (!tab)
-		return NULL;
+		return -1;
 
 	var = &tab->con[con];
 
+	if (var->is_row && (var->index < 0 || var->index < tab->n_redundant))
+		isl_die(tab->mat->ctx, isl_error_invalid,
+			"cannot relax redundant constraint", return -1);
+	if (!var->is_row && (var->index < 0 || var->index < tab->n_dead))
+		isl_die(tab->mat->ctx, isl_error_invalid,
+			"cannot relax dead constraint", return -1);
+
 	if (!var->is_row && !max_is_manifestly_unbounded(tab, var))
 		if (to_row(tab, var, 1) < 0)
-			goto error;
+			return -1;
 	if (!var->is_row && !min_is_manifestly_unbounded(tab, var))
 		if (to_row(tab, var, -1) < 0)
-			goto error;
+			return -1;
 
 	if (var->is_row) {
 		isl_int_add(tab->mat->row[var->index][1],
 		    tab->mat->row[var->index][1], tab->mat->row[var->index][0]);
 		if (restore_row(tab, var) < 0)
-			goto error;
+			return -1;
 	} else {
 		int i;
+		unsigned off = 2 + tab->M;
 
 		for (i = 0; i < tab->n_row; ++i) {
 			if (isl_int_is_zero(tab->mat->row[i][off + var->index]))
@@ -2570,12 +2679,90 @@ struct isl_tab *isl_tab_relax(struct isl_tab *tab, int con)
 	}
 
 	if (isl_tab_push_var(tab, isl_tab_undo_relax, var) < 0)
-		goto error;
+		return -1;
 
-	return tab;
-error:
-	isl_tab_free(tab);
-	return NULL;
+	return 0;
+}
+
+/* Replace the variable v at position "pos" in the tableau "tab"
+ * by v' = v + shift.
+ *
+ * If the variable is in a column, then we first check if we can
+ * simply plug in v = v' - shift.  The effect on a row with
+ * coefficient f/d for variable v is that the constant term c/d
+ * is replaced by (c - f * shift)/d.  If shift is positive and
+ * f is negative for each row that needs to remain non-negative,
+ * then this is clearly safe.  In other words, if the minimum of v
+ * is manifestly unbounded, then we can keep v in a column position.
+ * Otherwise, we can pivot it down to a row.
+ * Similarly, if shift is negative, we need to check if the maximum
+ * of is manifestly unbounded.
+ *
+ * If the variable is in a row (from the start or after pivoting),
+ * then the constant term c/d is replaced by (c + d * shift)/d.
+ */
+int isl_tab_shift_var(struct isl_tab *tab, int pos, isl_int shift)
+{
+	struct isl_tab_var *var;
+
+	if (!tab)
+		return -1;
+	if (isl_int_is_zero(shift))
+		return 0;
+
+	var = &tab->var[pos];
+	if (!var->is_row) {
+		if (isl_int_is_neg(shift)) {
+			if (!max_is_manifestly_unbounded(tab, var))
+				if (to_row(tab, var, 1) < 0)
+					return -1;
+		} else {
+			if (!min_is_manifestly_unbounded(tab, var))
+				if (to_row(tab, var, -1) < 0)
+					return -1;
+		}
+	}
+
+	if (var->is_row) {
+		isl_int_addmul(tab->mat->row[var->index][1],
+				shift, tab->mat->row[var->index][0]);
+	} else {
+		int i;
+		unsigned off = 2 + tab->M;
+
+		for (i = 0; i < tab->n_row; ++i) {
+			if (isl_int_is_zero(tab->mat->row[i][off + var->index]))
+				continue;
+			isl_int_submul(tab->mat->row[i][1],
+				    shift, tab->mat->row[i][off + var->index]);
+		}
+
+	}
+
+	return 0;
+}
+
+/* Remove the sign constraint from constraint "con".
+ *
+ * If the constraint variable was originally marked non-negative,
+ * then we make sure we mark it non-negative again during rollback.
+ */
+int isl_tab_unrestrict(struct isl_tab *tab, int con)
+{
+	struct isl_tab_var *var;
+
+	if (!tab)
+		return -1;
+
+	var = &tab->con[con];
+	if (!var->is_nonneg)
+		return 0;
+
+	var->is_nonneg = 0;
+	if (isl_tab_push_var(tab, isl_tab_undo_unrestrict, var) < 0)
+		return -1;
+
+	return 0;
 }
 
 int isl_tab_select_facet(struct isl_tab *tab, int con)
@@ -2679,6 +2866,106 @@ int isl_tab_detect_implicit_equalities(struct isl_tab *tab)
 	return 0;
 }
 
+/* Update the element of row_var or col_var that corresponds to
+ * constraint tab->con[i] to a move from position "old" to position "i".
+ */
+static int update_con_after_move(struct isl_tab *tab, int i, int old)
+{
+	int *p;
+	int index;
+
+	index = tab->con[i].index;
+	if (index == -1)
+		return 0;
+	p = tab->con[i].is_row ? tab->row_var : tab->col_var;
+	if (p[index] != ~old)
+		isl_die(tab->mat->ctx, isl_error_internal,
+			"broken internal state", return -1);
+	p[index] = ~i;
+
+	return 0;
+}
+
+/* Rotate the "n" constraints starting at "first" to the right,
+ * putting the last constraint in the position of the first constraint.
+ */
+static int rotate_constraints(struct isl_tab *tab, int first, int n)
+{
+	int i, last;
+	struct isl_tab_var var;
+
+	if (n <= 1)
+		return 0;
+
+	last = first + n - 1;
+	var = tab->con[last];
+	for (i = last; i > first; --i) {
+		tab->con[i] = tab->con[i - 1];
+		if (update_con_after_move(tab, i, i - 1) < 0)
+			return -1;
+	}
+	tab->con[first] = var;
+	if (update_con_after_move(tab, first, last) < 0)
+		return -1;
+
+	return 0;
+}
+
+/* Make the equalities that are implicit in "bmap" but that have been
+ * detected in the corresponding "tab" explicit in "bmap" and update
+ * "tab" to reflect the new order of the constraints.
+ *
+ * In particular, if inequality i is an implicit equality then
+ * isl_basic_map_inequality_to_equality will move the inequality
+ * in front of the other equality and it will move the last inequality
+ * in the position of inequality i.
+ * In the tableau, the inequalities of "bmap" are stored after the equalities
+ * and so the original order
+ *
+ *		E E E E E A A A I B B B B L
+ *
+ * is changed into
+ *
+ *		I E E E E E A A A L B B B B
+ *
+ * where I is the implicit equality, the E are equalities,
+ * the A inequalities before I, the B inequalities after I and
+ * L the last inequality.
+ * We therefore need to rotate to the right two sets of constraints,
+ * those up to and including I and those after I.
+ *
+ * If "tab" contains any constraints that are not in "bmap" then they
+ * appear after those in "bmap" and they should be left untouched.
+ *
+ * Note that this function leaves "bmap" in a temporary state
+ * as it does not call isl_basic_map_gauss.  Calling this function
+ * is the responsibility of the caller.
+ */
+__isl_give isl_basic_map *isl_tab_make_equalities_explicit(struct isl_tab *tab,
+	__isl_take isl_basic_map *bmap)
+{
+	int i;
+
+	if (!tab || !bmap)
+		return isl_basic_map_free(bmap);
+	if (tab->empty)
+		return bmap;
+
+	for (i = bmap->n_ineq - 1; i >= 0; --i) {
+		if (!isl_tab_is_equality(tab, bmap->n_eq + i))
+			continue;
+		isl_basic_map_inequality_to_equality(bmap, i);
+		if (rotate_constraints(tab, 0, tab->n_eq + i + 1) < 0)
+			return isl_basic_map_free(bmap);
+		if (rotate_constraints(tab, tab->n_eq + i + 1,
+					bmap->n_ineq - i) < 0)
+			return isl_basic_map_free(bmap);
+		tab->n_eq++;
+	}
+
+	return bmap;
+}
+
 static int con_is_redundant(struct isl_tab *tab, struct isl_tab_var *var)
 {
 	if (!tab)
@@ -2874,10 +3161,21 @@ enum isl_lp_result isl_tab_min(struct isl_tab *tab,
 	return res;
 }
 
+/* Is the constraint at position "con" marked as being redundant?
+ * If it is marked as representing an equality, then it is not
+ * considered to be redundant.
+ * Note that isl_tab_mark_redundant marks both the isl_tab_var as
+ * redundant and moves the corresponding row into the first
+ * tab->n_redundant positions (or removes the row, assigning it index -1),
+ * so the final test is actually redundant itself.
+ */
 int isl_tab_is_redundant(struct isl_tab *tab, int con)
 {
 	if (!tab)
 		return -1;
+	if (con < 0 || con >= tab->n_con)
+		isl_die(isl_tab_get_ctx(tab), isl_error_invalid,
+			"position out of bounds", return -1);
 	if (tab->con[con].is_zero)
 		return 0;
 	if (tab->con[con].is_redundant)
@@ -2929,6 +3227,21 @@ static int unrelax(struct isl_tab *tab, struct isl_tab_var *var)
 	return 0;
 }
 
+/* Undo the operation performed by isl_tab_unrestrict.
+ *
+ * In particular, mark the variable as being non-negative and make
+ * sure the sample value respects this constraint.
+ */
+static int ununrestrict(struct isl_tab *tab, struct isl_tab_var *var)
+{
+	var->is_nonneg = 1;
+
+	if (var->is_row && restore_row(tab, var) < -1)
+		return -1;
+
+	return 0;
+}
+
 static int perform_undo_var(struct isl_tab *tab, struct isl_tab_undo *undo) WARN_UNUSED;
 static int perform_undo_var(struct isl_tab *tab, struct isl_tab_undo *undo)
 {
@@ -2953,8 +3266,7 @@ static int perform_undo_var(struct isl_tab *tab, struct isl_tab_undo *undo)
 	case isl_tab_undo_allocate:
 		if (undo->u.var_index >= 0) {
 			isl_assert(tab->mat->ctx, !var->is_row, return -1);
-			drop_col(tab, var->index);
-			break;
+			return drop_col(tab, var->index);
 		}
 		if (!var->is_row) {
 			if (!max_is_manifestly_unbounded(tab, var)) {
@@ -2967,10 +3279,11 @@ static int perform_undo_var(struct isl_tab *tab, struct isl_tab_undo *undo)
 				if (to_row(tab, var, 0) < 0)
 					return -1;
 		}
-		drop_row(tab, var->index);
-		break;
+		return drop_row(tab, var->index);
 	case isl_tab_undo_relax:
 		return unrelax(tab, var);
+	case isl_tab_undo_unrestrict:
+		return ununrestrict(tab, var);
 	default:
 		isl_die(tab->mat->ctx, isl_error_internal,
 			"perform_undo_var called on invalid undo record",
@@ -2999,7 +3312,7 @@ static int restore_basis(struct isl_tab *tab, int *col_var)
 	unsigned off = 2 + tab->M;
 
 	extra = isl_alloc_array(tab->mat->ctx, int, tab->n_col);
-	if (!extra)
+	if (tab->n_col && !extra)
 		goto error;
 	for (i = 0; i < tab->n_col; ++i) {
 		for (j = 0; j < tab->n_col; ++j)
@@ -3062,6 +3375,9 @@ static int perform_undo(struct isl_tab *tab, struct isl_tab_undo *undo) WARN_UNU
 static int perform_undo(struct isl_tab *tab, struct isl_tab_undo *undo)
 {
 	switch (undo->type) {
+	case isl_tab_undo_rational:
+		tab->rational = 0;
+		break;
 	case isl_tab_undo_empty:
 		tab->empty = 0;
 		break;
@@ -3071,6 +3387,7 @@ static int perform_undo(struct isl_tab *tab, struct isl_tab_undo *undo)
 	case isl_tab_undo_zero:
 	case isl_tab_undo_allocate:
 	case isl_tab_undo_relax:
+	case isl_tab_undo_unrestrict:
 		return perform_undo_var(tab, undo);
 	case isl_tab_undo_bmap_eq:
 		return isl_basic_map_free_equality(tab->bmap, 1);
@@ -3231,12 +3548,21 @@ error:
 
 int isl_tab_track_bmap(struct isl_tab *tab, __isl_take isl_basic_map *bmap)
 {
+	bmap = isl_basic_map_cow(bmap);
 	if (!tab || !bmap)
 		goto error;
 
-	isl_assert(tab->mat->ctx, tab->n_eq == bmap->n_eq, return -1);
+	if (tab->empty) {
+		bmap = isl_basic_map_set_to_empty(bmap);
+		if (!bmap)
+			goto error;
+		tab->bmap = bmap;
+		return 0;
+	}
+
+	isl_assert(tab->mat->ctx, tab->n_eq == bmap->n_eq, goto error);
 	isl_assert(tab->mat->ctx,
-		    tab->n_con == bmap->n_eq + bmap->n_ineq, return -1);
+		    tab->n_con == bmap->n_eq + bmap->n_ineq, goto error);
 
 	tab->bmap = bmap;
 
