@@ -88,6 +88,10 @@ struct osl_body {
 };
 typedef struct osl_body   osl_body_t;
 typedef struct osl_body * osl_body_p;
+typedef struct osl_body const         osl_const_body_t;
+typedef struct osl_body       * const osl_body_const_p;
+typedef struct osl_body const *       osl_const_body_p;
+typedef struct osl_body const * const osl_const_body_const_p;
 
 
 /*---------------------------------------------------------------------------+
