@@ -64,6 +64,9 @@
 #ifndef OSL_MACROS_H
 # define OSL_MACROS_H
 
+# include "util.h"
+
+
 # define OSL_DEBUG                 0       // 1 for debug mode, 0 otherwise.
 
 # define OSL_URI_SCOP              "OpenScop"
@@ -149,6 +152,8 @@
            exit(1);                                                        \
          } while (0)
 
+# define OSL_overflow(msg) OSL_error(msg)
+
 # define OSL_malloc(ptr, type, size)                                       \
          do {                                                              \
            if (((ptr) = (type)malloc(size)) == NULL)                       \
@@ -164,7 +169,7 @@
 # define OSL_strdup(destination, source)                                   \
          do {                                                              \
            if (source != NULL) {                                           \
-             if (((destination) = strdup(source)) == NULL)                 \
+             if (((destination) = osl_util_strdup(source)) == NULL)                 \
                OSL_error("memory overflow");                               \
            }                                                               \
            else {                                                          \
