@@ -83,10 +83,12 @@ char * osl_util_read_string(FILE *, char **);
 char * osl_util_read_line(FILE *, char **);
 char * osl_util_read_tag(FILE *, char **);
 char * osl_util_read_tail(FILE *);
-char * osl_util_read_uptotag(FILE *, char *);
-char * osl_util_read_uptoendtag(FILE *, char *);
+char * osl_util_read_uptoflag(FILE *, char **, char *);
+char * osl_util_read_uptotag(FILE *, char **, char *);
+char * osl_util_read_uptoendtag(FILE *, char **, char *);
 char * osl_util_tag_content(char *, char *);
 void   osl_util_safe_strcat(char **, char *, int *);
+char * osl_util_strdup(char const *);
 int    osl_util_get_precision();
 void   osl_util_print_provided(FILE *, int, char *);
 char * osl_util_identifier_substitution(char *, char **);
