@@ -408,19 +408,21 @@ void osl_generic_add(osl_generic_p * list, osl_generic_p generic) {
  */
 void osl_generic_remove_node(osl_generic_p * list, osl_generic_p generic) {
 
-  osl_generic_p tmp = *list;
+  osl_generic_p tmp = NULL;
   
   if (generic != NULL) {
 
     if (*list != NULL) {
       //target is the first element of list
-      if(tmp==generic){
+      if(*list==generic){
         *list = generic->next;
         generic->next=NULL; //free below removes the whole list!
         osl_generic_free(generic); 
+        return;
       }
 
       //find target
+      tmp = *list;
       while (tmp->next!=generic && tmp->next != NULL)
         tmp = tmp->next;
 
@@ -643,7 +645,7 @@ int osl_generic_equal(osl_generic_p x1, osl_generic_p x2) {
  * \param[in] URI The URI value to test.
  * \return 1 if x has the provided URI, 0 otherwise.
  */
-int osl_generic_has_URI(osl_generic_p x, char * URI) {
+int osl_generic_has_URI(osl_const_generic_const_p x, char const * const URI) {
 
   if ((x == NULL) ||
       (x->interface == NULL) ||
@@ -664,7 +666,7 @@ int osl_generic_has_URI(osl_generic_p x, char * URI) {
  * \param[in] URI The URI of the generic we are looking for.
  * \return The first generic of the requested URI in the list.
  */
-void * osl_generic_lookup(osl_generic_p x, char * URI) {
+void * osl_generic_lookup(osl_generic_p x, char const * const URI) {
   while (x != NULL) {
     if (osl_generic_has_URI(x, URI))
       return x->data;
