@@ -190,7 +190,6 @@ void osl_scop_dump(FILE * file, osl_scop_p scop) {
  * \param[in] scop The scop (list) we have to generate names for.
  * \return A set of generated names for the input scop dimensions.
  */
-static
 osl_names_p osl_scop_names(osl_scop_p scop) {
   int nb_parameters = OSL_UNDEFINED;
   int nb_iterators  = OSL_UNDEFINED;
@@ -462,7 +461,7 @@ osl_scop_p osl_scop_pread(FILE * file, osl_interface_p registry,
       OSL_warning("uninterpreted information (after language)");
 
     if (language != NULL) {
-      scop->language = strdup(language->string[0]);
+      OSL_strdup(scop->language, language->string[0]);
       osl_strings_free(language);
     }
 
@@ -614,8 +613,8 @@ void osl_scop_add(osl_scop_p * location, osl_scop_p scop) {
  * \param[in] scop The first element of the scop list.
  * \return The number of scops in the scop list.
  */
-int osl_scop_number(osl_scop_p scop) {
-  int number = 0;
+size_t osl_scop_number(osl_scop_p scop) {
+  size_t number = 0;
 
   while (scop != NULL) {
     number++;
@@ -641,7 +640,7 @@ osl_scop_p osl_scop_clone(osl_scop_p scop) {
     node                 = osl_scop_malloc();
     node->version        = scop->version;
     if (scop->language != NULL)
-      node->language     = strdup(scop->language);
+      OSL_strdup(node->language, scop->language);
     node->context        = osl_relation_clone(scop->context);
     node->parameters     = osl_generic_clone(scop->parameters);
     node->statement      = osl_statement_clone(scop->statement);
