@@ -130,7 +130,7 @@ void osl_statement_idump(FILE * file, osl_statement_p statement, int level) {
     osl_relation_list_idump(file, statement->access, level + 1);
 
     // Print the original body expression.
-    osl_generic_idump(file, statement->body, level + 1);
+    osl_generic_idump(file, statement->extension, level + 1);
 
     statement = statement->next;
     number++;
@@ -198,9 +198,12 @@ osl_names_p osl_statement_names(osl_statement_p statement) {
  */
 void osl_statement_pprint(FILE * file, osl_statement_p statement,
                           osl_names_p names) {
-  int nb_relations, number = 1;
+  size_t nb_relations;
+  int number = 1;
   int generated_names = 0;
   int iterators_backedup = 0;
+  int nb_ext = 0;
+  osl_body_p body = NULL;
   osl_strings_p iterators_backup = NULL;
 
   // Generate the dimension names if necessary and replace iterators with
@@ -212,11 +215,11 @@ void osl_statement_pprint(FILE * file, osl_statement_p statement,
 
   while (statement != NULL) {
     // If possible, replace iterator names with statement iterator names.
-    if (osl_generic_has_URI(statement->body, OSL_URI_BODY) &&
-        (((osl_body_p)(statement->body->data))->iterators != NULL)) {
+    body = (osl_body_p)osl_generic_lookup(statement->extension, OSL_URI_BODY);
+    if (body && body->iterators != NULL) {
       iterators_backedup = 1;
       iterators_backup = names->iterators;
-      names->iterators = ((osl_body_p)(statement->body->data))->iterators;
+      names->iterators = body->iterators;
     }
 
     nb_relations = 0;
@@ -232,7 +235,7 @@ void osl_statement_pprint(FILE * file, osl_statement_p statement,
       nb_relations ++;
     nb_relations += osl_relation_list_count(statement->access); 
 
-    fprintf(file, "%d\n\n", nb_relations);
+    fprintf(file, "%lu\n\n", nb_relations);
 
     fprintf(file, "# ---------------------------------------------- ");
     fprintf(file, "%2d.1 Domain\n", number);
@@ -249,16 +252,100 @@ void osl_statement_pprint(FILE * file, osl_statement_p statement,
     osl_relation_list_pprint_elts(file, statement->access, names);
     fprintf(file, "\n");
 
+    fprintf(file, "# ---------------------------------------------- ");
+    fprintf(file, "%2d.4 Statement Extensions\n", number);
+    fprintf(file, "# Number of Statement Extensions\n");
+    nb_ext = osl_generic_number(statement->extension);
+    fprintf(file, "%d\n", nb_ext);
+    if(nb_ext>0)
+      osl_generic_print(file, statement->extension);
+
+    fprintf(file, "\n");
+
+    // If necessary, switch back iterator names.
+    if (iterators_backedup) {
+      iterators_backedup = 0;
+      names->iterators = iterators_backup;
+    }
+
+    statement = statement->next;
+    number++;
+  }
+
+  if (generated_names)
+    osl_names_free(names);
+}
+
+
+/**
+ * osl_statement_pprint_scoplib function:
+ * this function pretty-prints the content of an osl_statement_t structure
+ * (*statement) into a file (file, possibly stdout) in the SCoPLib format.
+ * \param[in] file      The file where the information has to be printed.
+ * \param[in] statement The statement whose information has to be printed.
+ * \param[in] names     The names of the constraint columns for comments. 
+ */
+void osl_statement_pprint_scoplib(FILE * file, osl_statement_p statement,
+                          osl_names_p names) {
+  int number = 1;
+  int generated_names = 0;
+  int iterators_backedup = 0;
+  osl_body_p body = NULL;
+  osl_strings_p iterators_backup = NULL;
+  int add_fakeiter;
+
+  // Generate the dimension names if necessary and replace iterators with
+  // statement iterators if possible.
+  if (names == NULL) {
+    generated_names = 1;
+    names = osl_statement_names(statement);
+  }
+
+  while (statement != NULL) {
+    // If possible, replace iterator names with statement iterator names.
+    body = (osl_body_p)osl_generic_lookup(statement->extension, OSL_URI_BODY);
+    if (body && body->iterators != NULL) {
+      iterators_backedup = 1;
+      iterators_backup = names->iterators;
+      names->iterators = body->iterators;
+    }
+    
+    add_fakeiter = statement->domain->nb_rows == 0 &&
+                   statement->scattering->nb_rows == 1;
+    
+    fprintf(file, "# =============================================== ");
+    fprintf(file, "Statement %d\n", number);
+    
+    fprintf(file, "# ---------------------------------------------- ");
+    fprintf(file, "%2d.1 Domain\n", number);
+    fprintf(file, "# Iteration domain\n");
+    osl_relation_pprint_scoplib(file, statement->domain, names, 1, add_fakeiter);
+    fprintf(file, "\n");
+
+    fprintf(file, "# ---------------------------------------------- ");
+    fprintf(file, "%2d.2 Scattering\n", number);
+    fprintf(file,"# Scattering function is provided\n1\n");
+    osl_relation_pprint_scoplib(file, statement->scattering, names, 0,
+                                add_fakeiter);
+    fprintf(file, "\n");
+
+    fprintf(file, "# ---------------------------------------------- ");
+    fprintf(file, "%2d.3 Access\n", number);
+    fprintf(file,"# Access informations are provided\n1\n");
+    
+    osl_relation_list_pprint_access_array_scoplib(file, statement->access,
+                                                  names, add_fakeiter);
+    fprintf(file, "\n");
+
     fprintf(file, "# ---------------------------------------------- ");
     fprintf(file, "%2d.4 Body\n", number);
-    if (statement->body != NULL) {
-      fprintf(file, "# Statement body is provided\n");
-      fprintf(file, "1\n");
-      osl_generic_print(file, statement->body);
+    if (body != NULL) {
+      fprintf(file, "# Statement body is provided\n1\n");
+      osl_body_print_scoplib(file, body);
+      body = NULL; //re-initialize for next statement
     }
     else {
-      fprintf(file, "# Statement body is not provided\n");
-      fprintf(file, "0\n");
+      fprintf(file, "# Statement body is not provided\n0\n");
     }
 
     fprintf(file, "\n");
@@ -309,7 +396,7 @@ static
 void osl_statement_dispatch(osl_statement_p stmt, osl_relation_list_p list) {
   osl_relation_list_p domain_list; 
   osl_relation_list_p scattering_list; 
-  int nb_domains, nb_scattering, nb_accesses;
+  size_t nb_domains, nb_scattering, nb_accesses;
 
   // Domain.
   domain_list = osl_relation_list_filter(list, OSL_TYPE_DOMAIN);
@@ -366,6 +453,8 @@ osl_statement_p osl_statement_pread(FILE * file, osl_interface_p registry,
                                     int precision) {
   osl_statement_p stmt = osl_statement_malloc();
   osl_relation_list_p list;
+  osl_generic_p new = NULL;
+  int i, nb_ext = 0;
 
   if (file) {
     // Read all statement relations.
@@ -374,9 +463,12 @@ osl_statement_p osl_statement_pread(FILE * file, osl_interface_p registry,
     // Store relations at the right place according to their type.
     osl_statement_dispatch(stmt, list);
 
-    // Read the body information.
-    if (osl_util_read_int(file, NULL) > 0)
-      stmt->body = osl_generic_read_one(file, registry); 
+    // Read the Extensions
+    nb_ext = osl_util_read_int(file, NULL);
+    for (i=0; i<nb_ext; i++) {
+      new = osl_generic_read_one(file, registry); 
+      osl_generic_add(&stmt->extension, new);
+    }
   }
 
   return stmt;
@@ -420,7 +512,7 @@ osl_statement_p osl_statement_malloc() {
   statement->domain     = NULL;
   statement->scattering = NULL;
   statement->access     = NULL;
-  statement->body       = NULL;
+  statement->extension  = NULL;
   statement->next       = NULL;
 
   return statement;
@@ -441,7 +533,7 @@ void osl_statement_free(osl_statement_p statement) {
     osl_relation_free(statement->domain);
     osl_relation_free(statement->scattering);
     osl_relation_list_free(statement->access);
-    osl_generic_free(statement->body);
+    osl_generic_free(statement->extension);
 
     free(statement);
     statement = next;
@@ -489,24 +581,25 @@ int osl_statement_number(osl_statement_p statement) {
 
 
 /**
- * osl_statement_clone function:
- * This functions builds and returns a "hard copy" (not a pointer copy) of an
- * osl_statement_t data structure provided as parameter.
- * \param[in] statement The pointer to the statement we want to clone.
- * \return A pointer to the clone of the statement provided as parameter.
+ * osl_statement_nclone function:
+ * This function builds and returns a "hard copy" (not a pointer copy) of the
+ * n first elements of an osl_statement_t list.
+ * \param statement The pointer to the statement structure we want to clone.
+ * \param n         The number of nodes we want to copy (-1 for infinity).
+ * \return The clone of the n first nodes of the statement list.
  */
-osl_statement_p osl_statement_clone(osl_statement_p statement) {
-  int first = 1;
+osl_statement_p osl_statement_nclone(osl_statement_p statement, int n) {
+  int first = 1, i = 0;
   osl_statement_p clone = NULL, node, previous = NULL;
 
-  while (statement != NULL) {
+  while ((statement != NULL) && ((n == -1) || (i < n))) {
     node             = osl_statement_malloc();
     node->domain     = osl_relation_clone(statement->domain);
     node->scattering = osl_relation_clone(statement->scattering);
     node->access     = osl_relation_list_clone(statement->access);
-    node->body       = osl_generic_clone(statement->body);
+    node->extension  = osl_generic_clone(statement->extension);
     node->next       = NULL;
-    
+
     if (first) {
       first = 0;
       clone = node;
@@ -517,6 +610,7 @@ osl_statement_p osl_statement_clone(osl_statement_p statement) {
       previous = previous->next;
     }
 
+    i++;
     statement = statement->next;
   }
 
@@ -524,6 +618,18 @@ osl_statement_p osl_statement_clone(osl_statement_p statement) {
 }
 
 
+/**
+ * osl_statement_clone function:
+ * This functions builds and returns a "hard copy" (not a pointer copy) of an
+ * osl_statement_t data structure provided as parameter.
+ * \param[in] statement The pointer to the statement we want to clone.
+ * \return A pointer to the clone of the statement provided as parameter.
+ */
+osl_statement_p osl_statement_clone(osl_statement_p statement) {
+  return osl_statement_nclone(statement, -1);
+}
+
+
 /**
  * osl_statement_equal function:
  * this function returns true if the two statements provided as parameters
@@ -565,7 +671,7 @@ int osl_statement_equal(osl_statement_p s1, osl_statement_p s2) {
     return 0;
   }
 
-  if (!osl_generic_equal(s1->body, s2->body)) {
+  if (!osl_generic_equal(s1->extension, s2->extension)) {
     OSL_info("statement bodies are not the same"); 
     return 0;
   }
@@ -587,6 +693,7 @@ int osl_statement_equal(osl_statement_p s1, osl_statement_p s2) {
 int osl_statement_integrity_check(osl_statement_p statement,
                                   int expected_nb_parameters) {
   int expected_nb_iterators;
+  osl_body_p body = NULL;
 
   while (statement != NULL) {
     // Check the domain.
@@ -623,11 +730,10 @@ int osl_statement_integrity_check(osl_statement_p statement,
     }
 
     // Check the statement body.
+    body = (osl_body_p)osl_generic_lookup(statement->extension, OSL_URI_BODY);
     if ((expected_nb_iterators != OSL_UNDEFINED) &&
-        (osl_generic_has_URI(statement->body, OSL_URI_BODY)) &&
-        (((osl_body_p)(statement->body->data))->iterators != NULL) &&
-        (expected_nb_iterators != osl_strings_size(
-            ((osl_body_p)(statement->body->data))->iterators))) {
+        body && body->iterators != NULL &&
+        ((size_t)expected_nb_iterators != osl_strings_size(body->iterators))) {
       OSL_warning("unexpected number of original iterators");
       return 0;
     }
@@ -719,3 +825,27 @@ void osl_statement_get_attributes(osl_statement_p statement,
   }
 }
 
+
+/**
+ * osl_statement_get_body function:
+ * this function returns the body of the statement.
+ * \param[in] statement The statement to search the body.
+ * \return the body if found, NULL otherwise.
+ */
+osl_body_p osl_statement_get_body(osl_statement_p statement) {
+  osl_body_p body;
+  osl_extbody_p ebody;
+
+  if (statement == NULL || statement->extension == NULL) {
+    return NULL;
+  }
+
+  body = (osl_body_p)osl_generic_lookup(statement->extension, OSL_URI_BODY);
+  if (body != NULL)
+    return body;
+  ebody = (osl_extbody_p)osl_generic_lookup(statement->extension,
+                                                           OSL_URI_EXTBODY);
+  if (ebody != NULL)
+    return ebody->body;
+  return NULL;
+}
