@@ -129,7 +129,7 @@ void osl_strings_dump(FILE * file, osl_strings_p strings) {
  * \return A string containing the OpenScop dump of the strings structure.
  */
 char * osl_strings_sprint(osl_strings_p strings) {
-  int i;
+  size_t i;
   int high_water_mark = OSL_MAX_STRING;
   char * string = NULL;
   char buffer[OSL_MAX_STRING];
@@ -239,6 +239,7 @@ osl_strings_p osl_strings_sread(char ** input) {
 
     // Build the strings structure
     strings = osl_strings_malloc();
+    free(strings->string);
     strings->string = string;
   }
 
@@ -283,7 +284,8 @@ osl_strings_p osl_strings_malloc() {
   osl_strings_p strings;
 
   OSL_malloc(strings, osl_strings_p, sizeof(osl_strings_t));
-  strings->string = NULL;
+  OSL_malloc(strings->string, char**, sizeof(char*));
+  strings->string[0] = NULL;
 
   return strings;
 }
@@ -334,6 +336,7 @@ osl_strings_p osl_strings_clone(osl_strings_p strings) {
   if ((nb_strings = osl_strings_size(strings)) == 0)
     return clone;
 
+  free(clone->string);
   OSL_malloc(clone->string, char **, (nb_strings + 1) * sizeof(char *));
   clone->string[nb_strings] = NULL;
   for (i = 0; i < nb_strings; i++)
@@ -342,6 +345,36 @@ osl_strings_p osl_strings_clone(osl_strings_p strings) {
   return clone;
 }
 
+/**
+ * osl_strings_find function.
+ * this function finds the string in the strings.
+ * \param[in,out] strings The strings structure.
+ * \param[in]     string  The string to find in strings.
+ * \return the index where is the string, osl_strings_size if not found
+ */
+size_t osl_strings_find(osl_strings_p strings, char const * const string) {
+  size_t i;
+  for (i = 0; i < osl_strings_size(strings); ++i) {
+    if (strcmp(strings->string[i], string) == 0) { return i; }
+  }
+  return i;
+}
+
+
+/**
+ * osl_strings_add function.
+ * this function adds a copy of the string in the strings.
+ * \param[in,out] strings The strings structure.
+ * \param[in]     string  The string to add in strings.
+ */
+void osl_strings_add(osl_strings_p strings, char const * const string) {
+  size_t original_size = osl_strings_size(strings);
+  OSL_realloc(strings->string, char**, sizeof(char*) * (original_size + 1 + 1));
+  strings->string[original_size + 1] = NULL;
+  strings->string[original_size] = malloc(sizeof(char) * (strlen(string) + 1));
+  strcpy(strings->string[original_size], string);
+}
+
 
 /**
  * osl_strings_equal function:
@@ -352,7 +385,7 @@ osl_strings_p osl_strings_clone(osl_strings_p strings) {
  * \return 1 if s1 and s2 are the same (content-wise), 0 otherwise.
  */
 int osl_strings_equal(osl_strings_p s1, osl_strings_p s2) {
-  int i, nb_s1;
+  size_t i, nb_s1;
   
   if (s1 == s2)
     return 1;
@@ -377,8 +410,8 @@ int osl_strings_equal(osl_strings_p s1, osl_strings_p s2) {
  * \param[in] strings The strings structure we need to know the size.
  * \return The number of strings in the strings structure.
  */
-int osl_strings_size(osl_strings_p strings) {
-  int size = 0;
+size_t osl_strings_size(osl_const_strings_const_p strings) {
+  size_t size = 0;
 
   if ((strings != NULL) && (strings->string != NULL)) {
     while (strings->string[size] != NULL) {
@@ -399,7 +432,7 @@ int osl_strings_size(osl_strings_p strings) {
  */
 osl_strings_p osl_strings_encapsulate(char * string) {
   osl_strings_p capsule = osl_strings_malloc();
-  
+  free(capsule->string);
   OSL_malloc(capsule->string, char **, 2 * sizeof(char *));
   capsule->string[0] = string;
   capsule->string[1] = NULL;
@@ -417,7 +450,7 @@ osl_strings_p osl_strings_encapsulate(char * string) {
 osl_interface_p osl_strings_interface() {
   osl_interface_p interface = osl_interface_malloc();
   
-  interface->URI    = strdup(OSL_URI_STRINGS);
+  OSL_strdup(interface->URI, OSL_URI_STRINGS);
   interface->idump  = (osl_idump_f)osl_strings_idump;
   interface->sprint = (osl_sprint_f)osl_strings_sprint;
   interface->sread  = (osl_sread_f)osl_strings_sread;
@@ -450,13 +483,14 @@ osl_strings_p osl_strings_generate(char * prefix, int nb_strings) {
     strings[nb_strings] = NULL;
     for (i = 0; i < nb_strings; i++) {
       sprintf(buff, "%s%d", prefix, i + 1);
-      strings[i] = strdup(buff);
+      OSL_strdup(strings[i], buff);
       if (strings[i] == NULL)
         OSL_error("memory overflow");
     }
   }
 
   generated = osl_strings_malloc();
+  free(generated->string);
   generated->string = strings;
   return generated;
 }
