@@ -312,14 +312,7 @@ int pprint_osl_body(struct cloogoptions *options, FILE *dst,
       stmt = stmt->next;
 
     /* Ensure it has a printable body. */
-    body = NULL;
-    if (osl_generic_has_URI(stmt->body, OSL_URI_BODY)) {
-      body = stmt->body->data;
-    } else if (osl_generic_has_URI(stmt->body, OSL_URI_EXTBODY)) {
-      if (stmt->body->data != NULL) {
-        body = ((osl_extbody_p)(stmt->body->data))->body;
-      }
-    }
+    body = osl_statement_get_body(stmt);
     if ((body != NULL) &&
         (body->expression != NULL) &&
         (body->iterators != NULL)) {
@@ -348,9 +341,33 @@ int pprint_osl_body(struct cloogoptions *options, FILE *dst,
   return 0;
 }
 
+/* pprint_parentheses_are_safer function:
+ * this function returns 1 if it decides that it would be safer to put
+ * parentheses around the clast_assignment when it is used as a macro
+ * parameter, 0 otherwise.
+ * \param[in] s Pointer to the clast_assignment to check.
+ * \return 1 if we should print parentheses around s, 0 otherwise.
+ */
+static int pprint_parentheses_are_safer(struct clast_assignment * s) {
+  /* Expressions of the form X = Y should not be used in macros, so we
+   * consider readability first for them and avoid parentheses.
+   * Also, expressions having only one term can live without parentheses.
+   */
+  if ((s->LHS) ||
+      (s->RHS->type == clast_expr_term) ||
+      ((s->RHS->type == clast_expr_red) &&
+       (((struct clast_reduction *)(s->RHS))->n == 1) &&
+       (((struct clast_reduction *)(s->RHS))->elts[0]->type ==
+        clast_expr_term)))
+    return 0;
+
+  return 1;
+}
+
 void pprint_user_stmt(struct cloogoptions *options, FILE *dst,
 		       struct clast_user_stmt *u)
 {
+    int parenthesis_to_close = 0;
     struct clast_stmt *t;
 
     if (pprint_osl_body(options, dst, u))
@@ -363,10 +380,21 @@ void pprint_user_stmt(struct cloogoptions *options, FILE *dst,
     fprintf(dst, "(");
     for (t = u->substitutions; t; t = t->next) {
 	assert(CLAST_STMT_IS_A(t, stmt_ass));
+        if (pprint_parentheses_are_safer((struct clast_assignment *)t)) {
+	  fprintf(dst, "(");
+          parenthesis_to_close = 1;
+        }
 	pprint_assignment(options, dst, (struct clast_assignment *)t);
-	if (t->next)
+	if (t->next) {
+            if (parenthesis_to_close) {
+	      fprintf(dst, ")");
+              parenthesis_to_close = 0;
+            }
 	    fprintf(dst, ",");
+        }
     }
+    if (parenthesis_to_close)
+      fprintf(dst, ")");
     fprintf(dst, ")");
     if (options->language != CLOOG_LANGUAGE_FORTRAN)
 	fprintf(dst, ";");
@@ -414,6 +442,10 @@ void pprint_for(struct cloogoptions *options, FILE *dst, int indent,
 		 struct clast_for *f)
 {
     if (options->language == CLOOG_LANGUAGE_C) {
+        if (f->time_var_name) {
+            fprintf(dst, "IF_TIME(%s_start = cloog_util_rtclock());\n",
+                    (f->time_var_name) ? f->time_var_name : "");
+        }
         if ((f->parallel & CLAST_PARALLEL_OMP) && !(f->parallel & CLAST_PARALLEL_MPI)) {
             if (f->LB) {
                 fprintf(dst, "lbp=");
@@ -537,6 +569,14 @@ void pprint_for(struct cloogoptions *options, FILE *dst, int indent,
 	fprintf(dst,"END DO\n") ; 
     else
 	fprintf(dst,"}\n") ; 
+
+    if (options->language == CLOOG_LANGUAGE_C) {
+        if (f->time_var_name) {
+            fprintf(dst, "IF_TIME(%s += cloog_util_rtclock() - %s_start);\n",
+                    (f->time_var_name) ? f->time_var_name : "",
+                    (f->time_var_name) ? f->time_var_name : "");
+        }
+    }
 }
 
 void pprint_stmt_list(struct cloogoptions *options, FILE *dst, int indent,
@@ -579,3 +619,9 @@ void clast_pprint(FILE *foo, struct clast_stmt *root,
 {
     pprint_stmt_list(options, foo, indent, root);
 }
+
+
+void clast_pprint_expr(struct cloogoptions *i, FILE *dst, struct clast_expr *e)
+{
+    pprint_expr(i, dst, e);
+}
